
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: PlatUtils.pm,v 1.29.8.1 2013/11/30 18:56:18 sgupta2 Exp $
#

package PlatUtils;

use base qw(Exporter);
our @EXPORT = qw(getLogDir
	getLockDir
	getVCSPaths
	getIPV6Prefix
	isSupportedAppPlat
	dumpToTmpFile
	removeTmpFiles
	getVCSHome
	PACKAGE_INSTALLED
	PACKAGE_NOT_INSTALLED
	SSH_DIR
	AUTHORIZED_KEYS_FILE
	);

use strict;
use warnings;
use Net::Domain qw(hostdomain);
use constant CONF_FILE => "supportedApp.conf";
use constant V1_LT_V2           => -1; # V1 < V2
use constant V1_EQ_V2           => 0;  # V1 == V2
use constant V1_HT_V2           => 1;  # V1 > V2
use constant PACKAGE_INSTALLED           => 0; # V1 < V2
use constant PACKAGE_NOT_INSTALLED           => 1;  # V1 == V2


my $LibPath1 = "";
BEGIN
{
	$LibPath1 = `find /opt/VRTSperl/lib/site_perl -name "5.*"`;
	chomp $LibPath1
}

use lib $LibPath1;
use Data::Dumper;
use XML::Simple;
use Net::IP qw(:PROC);
use POSIX;
use Constants;

my $llttabFileFullSpec = "/etc/llttab";
my $llthostFileFullSpec = "/etc/llthosts";
my $gabtabFileFullSpec = "/etc/gabtab";

my $vcshome = "/opt/VRTSvcs";
my $vcsvmwAdminDir = "$vcshome/portal/admin/";
my $logDir = "/var/VRTSvcs/log/";
my $lockDir = "/var/VRTSvcs/lock/";

# The path for .ssh folder is specific for linux. It will not work on AIX, Solaris.
use constant SSH_DIR => "/root/.ssh";
use constant AUTHORIZED_KEYS_FILE => SSH_DIR . "/authorized_keys";
my $keyFile = SSH_DIR . "/id_dsa";

#---------------------
# Subroutine section
#---------------------
sub getLogDir
{
	return $logDir;
}

sub getLockDir
{
	return $lockDir;
}


sub dumpToTmpFile
{
	my $fileContent = shift;

	my $timestamp = strftime '%Y_%m_%d_%H_%M_%S', localtime;

	my $range  = 100;
	my $random = int(rand($range));

	my $tmpFileName = "/tmp/vcsvmwTmpFile_$random" . "_" . $timestamp . "_" . $$;
	
	open FH, ">$tmpFileName";
	print FH $fileContent;
	close FH;

	return $tmpFileName;
}


sub removeTmpFiles
{
	`rm -rf /tmp/vcsvmwTmpFile_*`;
}


sub getVCSPaths
{
	my $hash = {};

	my $vcsBinHome = "$vcshome/bin";

	$hash -> { Home } = $vcshome;
	$hash -> { ConfigDir } = "/etc/VRTSvcs/conf/config";
	$hash -> { InstallDir } = "/opt/VRTS/install";
	$hash -> { MainCf } = "/etc/VRTSvcs/conf/config/main.cf";
	$hash -> { LogDir } = "/var/VRTSvcs/log";

	$hash -> { hagrp   } = "$vcsBinHome/hagrp";
	$hash -> { hares   } = "$vcsBinHome/hares";
	$hash -> { hatype  } = "$vcsBinHome/hatype";
	$hash -> { haclus  } = "$vcsBinHome/haclus";
	$hash -> { haconf  } = "$vcsBinHome/haconf";
	$hash -> { hasys   } = "$vcsBinHome/hasys";
	$hash -> { hastart } = "$vcsBinHome/hastart";
	$hash -> { hastop  } = "$vcsBinHome/hastop";
	$hash -> { vcsencrypt } = "$vcsBinHome/vcsencrypt";

	$hash -> { vxlicrep } = "/opt/VRTS/bin/vxlicrep";
	$hash -> { vxlicinst } = "/opt/VRTS/bin/vxlicinst";

	$hash -> { lltconfig } = "/sbin/lltconfig";

	$hash -> { llttab } = $llttabFileFullSpec;
	$hash -> { llthosts } = $llthostFileFullSpec;
	$hash -> { gabtab } = $gabtabFileFullSpec;

	$hash -> { AdminDir } = $vcsvmwAdminDir;
	$hash -> { notifysink } = $vcsvmwAdminDir . "notify_sink";

	$hash -> { xprtlc } = "/opt/VRTSsfmh/bin/xprtlc";

	return $hash;
}

sub getSysPaths
{
	my $hash = {};
	my $os = getOS ();

	# hash -> { OS }{ command } = <path_to_command>
	if ( $os =~ /Linux/ ) {
		my $redhatFile = "/etc/redhat-release";
		my $suseFile = "/etc/SuSE-release";
		if ( -f $redhatFile ) {
		# All command paths for Red Hat
			$hash -> { Linux }{ ip } = "/sbin/ip";
		} elsif ( -f $suseFile ) {
		# All command paths for SuSE
			$hash -> { Linux }{ ip } = "/sbin/ip";
		}
	}

	return $hash->{ $os };
}

sub exportPATHVaribale
{
	my $Path = "/opt/VRTSvcs/bin:" . $ENV { PATH };
	$ENV { PATH } = $Path;
}


sub getVCSHome
{
	return $vcshome;
}

sub getMessageDir
{
	return $vcsvmwAdminDir . "messsages/en/";
}


sub getHVConfDir
{
	return $vcsvmwAdminDir . "conf/";
}


sub getLLTTabFilePath
{	
	return $llttabFileFullSpec;
}


#----------------------------------------------------
# Readable name for unix as well as we do for windows
# instead of just using Config os name.
#----------------------------------------------------
sub getOS
{
	my $osName = "";

	if ( $^O =~ /lin/i ) {
		$osName = "Linux";
	} elsif ( $^O =~ /sol/i ) {
		$osName = "Solaris";
	} elsif ( $^O =~ /aix/i ) {
		$osName = "AIX";
	}

	return $osName;
}


sub getOSName
{
	my $osName = "";

	if ( $^O =~ /lin/i ) {

		my $redhatFile = "/etc/redhat-release";
		my $suseFile = "/etc/SuSE-release";
		if ( -f $redhatFile ) {
			$osName = `cat $redhatFile`;
		} elsif ( -f $suseFile ) {
			$osName = `cat $suseFile`;
			$osName =~ s/\n/ /g;
		}

		chomp $osName;

	} elsif ( $^O =~ /sol/i ) {
	} elsif ( $^O =~ /aix/i ) {
	}

	return $osName; # like Red Hat Linux Server Edition
}

sub getDomainName
{
	return hostdomain();
}


sub getOSArch
{
	my $op = `uname -m`;
	chomp $op;
	return $op;
}


sub getOSPADV
{
	my $arch      = getOSArch();
	my $osCaption = getOSName();
	my $distro    = "";

	if ( $^O =~ /lin/i ) {
		if ( $osCaption =~ /SUSE.*VERSION = (\d+)/i ) {
			$distro = "sles$1";
		}
		elsif ( $osCaption =~ /Enterprise Linux.*?(\d+)/i ) {
			$distro = "rhel$1";
		}
	} elsif ( $^O =~ /sol/i ) {
	} elsif ( $^O =~ /aix/i ) {
	}

	return $distro . "_" . $arch;
}


sub getOSBits
{
	my $ret = "32";

	my $op = `uname -m`;
	chomp $op;
	$ret = 64 if ($op =~ /x86_64/i);
	return $ret;
}


sub getNICList
{
	if ( $^O =~ /lin/i ) {
		return getNICList_lin();
	}
	elsif ( $^O =~ /sol/i ) {
	}
	elsif ( $^O =~ /aix/i ) {
	}
}

sub getNICList_lin
{
	my $nicListHash = {};
	my $nicID = 0;

	my $nic = "";
	my $nicHash = {};
	my $ipCount = 0;
	my $IPaddressHash = {};
	my %nicPrefixeHash = {};

	my $hashSysPaths = getSysPaths ();
	my $ip = $hashSysPaths -> { ip };

	open (IPA, "$ip a |");

	while ( <IPA> ) {
		if ( /^\S/ ) {
			my @nicInfo = split ( /: / );
			$nic = $nicInfo[1];
			next if ( $nic =~ /lo|sit/ );

			$nicHash -> { IPAddresses } = $IPaddressHash if defined ( (keys %$IPaddressHash)[0] );
			$nicListHash -> { $nicID++ } = $nicHash if defined ( (keys %$nicHash)[0] );
			$nicHash = {};
			$nicHash -> { Caption } = $nic;
			$nicHash -> { DHCPEnabled } = "";
			$ipCount = 0;
			$IPaddressHash = {};
		}
		else {
			next if ( $nic =~ /lo|sit/ );

			s/^\s+//;

			my %IPDetailsHash;
			my @nicInfo = split /\s+/;
			if ( $nicInfo[0] =~ /link\/ether/ ) {
				$nicHash -> { MACAddress } = $nicInfo[1];
			}
			elsif ( $nicInfo[0] =~ /inet6/ ) {
				my @ipInfo = split ( /\//, $nicInfo[1]);
				$IPDetailsHash {IPAddress} = $ipInfo[0];
				$IPDetailsHash {NetMask} = "";
				$IPDetailsHash {PrefixLen} = $ipInfo[1];
				$IPDetailsHash {IPVersion} = "IPv6";
				$IPaddressHash -> { $ipCount++ } = \%IPDetailsHash;
				my $prefix = getIPV6Prefix($ipInfo[0], $ipInfo[1]);
				$prefix = "$prefix/$ipInfo[1]";
				push(@{$nicPrefixeHash{$nicID}}, $prefix);
			}
			elsif ( $nicInfo[0] =~ /inet/ ) {
				my @ipInfo = split ( /\//, $nicInfo[1]) ;
				$IPDetailsHash {IPAddress} = $ipInfo[0];
				$IPDetailsHash {NetMask} = ip_bintoip ( ip_get_mask ( $ipInfo[1], 4 ), 4 );
				$IPDetailsHash {PrefixLen} = "";
				$IPDetailsHash {IPVersion} = "IPv4";
				$IPaddressHash -> { $ipCount++ } = \%IPDetailsHash;
			}

		}
	}

	$nicHash -> { IPAddresses } = $IPaddressHash;
	$nicListHash -> { $nicID++ } = $nicHash;
	foreach my $nicid (keys %$nicListHash) {
		$nicListHash->{$nicid}->{Prefixes} = $nicPrefixeHash{$nicid} if $nicPrefixeHash{$nicid};
	}

	return $nicListHash;
}

sub getConfiguredIPInfo
{
	my $configuredIPListHash = {};

	my $vcsobj = VCS -> new();
	my ($cmdRet, $cmdMsg, $clusRef);
	my $clusDetailsInputRef = ();
	my $sConfiguredVIPsXML = "";
	$clusDetailsInputRef -> { Resources }{ Types      } = [ "IP" ];
	$clusDetailsInputRef -> { Resources }{ Attributes } = [ "Address" ];
	($cmdRet, $cmdMsg, $clusRef) = $vcsobj -> getClusterDetails($clusDetailsInputRef);	
	if ( $cmdRet == CONST_FAILURE ) {
		Utils::vxlog("ERROR", "Error occurred while fetching resource details- [$cmdMsg]");
		goto END;
	}
	else {
		$configuredIPListHash = $clusRef -> { ResRef };
		Utils::vxlog("INFO", Dumper( $configuredIPListHash ));
	}

END:
	return $configuredIPListHash;
}

#
# Function description : This function takes the IP address
# and prefix length as input and returns the correspoding
# network address as output
#
# Input values : IP address and Preifx length
#
# Return value : Network address (prefix)
#

sub getIPV6Prefix
{
	my $ip = shift;
	my $prefixlen = shift;
	my $prefix = "";
	my $ipObject = new Net::IP ("$ip", 6) or die (Net::IP::Error());
	my $binary_ip = $ipObject->binip();
	my $prefix_in_bin = substr $binary_ip, 0, $prefixlen;
	my $numofzeros = 128 - length( $prefix_in_bin);
	$prefix_in_bin =~ s/(.*)$/$1.'0' x $numofzeros/e;
	my $hex = "";
	for (my $i = 0; $i < 128; $i = $i+4) {
		my $str = substr $prefix_in_bin, $i, 4;
		$hex = sprintf('%X', oct("0b$str"));
		if ($i && $i % 16 == 0) {
			$prefix = "$prefix".":";
		}
		$prefix = "$prefix$hex";
	}
	# Removing trailing zeros
	$prefix = Net::IP::ip_compress_address("$prefix", 6);
	return $prefix;
}


sub getVirtualizationPlatform
{	
	my $testVMwareFile = "/opt/VRTSsfmh/bin/testVMware";
	my $sVirtPlatform = "Non-VMware";

	if ( -x $testVMwareFile) {
		my ($fret, $fmsg, $iExitCode) = Utils::runCmd("\"$testVMwareFile\"", {doNotLogCmd => 1, doNotLogOutput => 1});
		if ( $iExitCode == 0 ) {
			$sVirtPlatform = "VMware";
		}
	}
	return $sVirtPlatform;
}


sub runCmdAsync
{
	
	my $cmd		= shift;
	my $cmdArgs = shift;
	my $cmdPath = shift; # Command full path required only on windows in Win32::Process API.

	my $subName = ( caller 0 )[3];
	Utils::vxlog("INFO", "Subroutine $subName called");

	my $ret = CONST_SUCCESS;
	my $msg = "";

	if(!defined $cmd)
	{
		Utils::vxlog("ERROR", "Incomplete arguments passed to runCmdAsync.");
		$ret = CONST_FAILURE;
		goto END;
	}

	$cmd = "$cmdArgs" if defined $cmdArgs;
	$cmd.= " &";
	
	Utils::vxlog("INFO", "Launching [$cmd] asynchronously.");
	my ($cmdRet, $op) = Utils::runCmd($cmd);
	if( $cmdRet != CONST_SUCCESS)
	{				
		Utils::vxlog("ERROR", "failed to launch $cmd");
		$ret = $cmdRet;
		goto END;
	}
	else
	{				
		Utils::vxlog("INFO", "successfully launched $cmd.");
		$ret = CONST_SUCCESS;
		goto END;
	}
END:
	return $ret;
}


sub isAgentPackageInstalled
{
	my $agentID = shift;
	my $agentVersion = shift;
	my $sMyName = ( caller 0 )[3];

#---------
# Announce
#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = 0;
	my $iRet = 1;
	my $packageName = "" ;

	my $confFile = PlatUtils::getHVConfDir() . CONF_FILE;
	no strict;
	do $confFile || die "can not read conf file [$confFile]";
	if( defined $AppConfig { $agentID } { Package } )
	{
		$packageName = $AppConfig { $agentID } { Package } ;
		if( ( defined $agentVersion) && (defined $packageName ) )
		{
			my ($ret, $op) = Utils::runCmd("rpm -q $packageName  --qf \"%{VERSION}\"");
			if($ret == 0)
			{
				my @lines = split(/\n/, $op);

				if ( defined $lines[0] and $lines[0] ne "" ) {
					$retAgentVersion = Utils::strTrim($lines[0]);
					my $logStr = "Received version number is [$retAgentVersion] .";
					Utils::vxlog("INFO", $logStr);
					$ret = compare_release($retAgentVersion,$agentVersion);
					if( $ret == V1_LT_V2 )
					{	
						$logStr = "Package [$packageName] with version [$agentVersion] or later is not installed for application [$agentID]. Currently installed package version is [$retAgentVersion]";
						Utils::vxlog("INFO", $logStr);
						$iRet = 1;
					}
					else
					{
						$logStr = "Package [$packageName] with version [$agentVersion] or later is installed for application [$agentID]. Currently installed version is [$retAgentVersion]";
						Utils::vxlog("INFO", $logStr);
						$iRet = 0;
					}
				}
				else
				{
					$iRet = 1;
					$logStr = "Package [$packageName] is not installed for application [$agentID]. Returning [$iRet]";
					Utils::vxlog("INFO", $logStr);
				}
			}
			else
			{
				$logStr = "Package [$packageName] is not installed for ID: [$agentID] ret=[$iRet]";
				Utils::vxlog("INFO", $logStr);
				$iRet = 1;
			}
		}
		elsif( defined $packageName  )
		{ 
			my ($ret, $op) = Utils::runCmd("rpm -q $packageName");
			if($ret == 0)
			{
				$logStr = "Package [$packageName] is installed for ID : $agentID [$iRet]";
				Utils::vxlog("INFO", $logStr);
				$iRet = 0 ;
			}
			else
			{
				$logStr = "Package [$packageName] is not installed for ID : $agentID [$iRet]";
				Utils::vxlog("INFO", $logStr);
				$iRet = 1;
			}
		}
	}
	else
	{
		$iRet = 1 ;
		$logStr = "Package name is not specified for ID in supportedApp.conf: $agentID [$iRet]";
		Utils::vxlog("INFO", $logStr);
	}
	return ($iRet,$packageName);

}


#----------------------------------------------------------
## compare_release function: This will comapre two release.
## Arg1 = $release1
## Arg2 = $release2
## Return:
## V1_EQ_V2 if $release1 == $release2
## V1_HT_V2 if $release1 > $release2
## V1_LT_V2 if $release1 < $release2
##----------------------------------------------------------
sub compare_release 
{
	my $release1 = shift;
	my $release2 = shift;
	my @version1;
	my @version2;
	if ($release1 eq $release2) {
		return V1_EQ_V2;
	}
	@version1 = split(/\./,$release1);
	@version2 = split(/\./,$release2);
	for (my $i = 0; $i < scalar@version1; $i++) {
		if ($version1[$i] == $version2[$i]) {
			next ;
		} elsif ($version1[$i] > $version2[$i]) {
			return V1_HT_V2;
		} else {
			return V1_LT_V2;
		}
	}
	return V1_EQ_V2;
}


sub isAcclibPackageInstalled
{
	my $agentID = shift ;
	my $sMyName = ( caller 0 )[3];

#---------
# Announce
#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);
	my $ret = 0;
	my $op = "";
	my $iRet = PACKAGE_NOT_INSTALLED;

	($ret, $op) = Utils::runCmd("rpm -qi VRTSacclib");

	if($ret == 0)
	{
		$logStr = "Package [VRTSacclib] is installed for ID : $agentID.";
		Utils::vxlog("INFO", $logStr);
		$iRet = PACKAGE_INSTALLED ;
	}
	else
	{
		$logStr = "Package [VRTSacclib] is not installed for ID : $agentID.";
		Utils::vxlog("INFO", $logStr);
		$iRet = PACKAGE_NOT_INSTALLED;
	}

	return $iRet;

}
    

sub hasPasswdFreeSSH
{
	my $dstHost = shift;
	my $sMyName = ( caller 0 )[3];

	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $cmd = "ssh -o NumberOfPasswordPrompts=0 -o StrictHostKeyChecking=no -C -l root \"$dstHost\" x=x";

	my ($ret, $msg, $cmdRet) = Utils::runCmd($cmd);
	if ( $ret != CONST_SUCCESS ) {
		Utils::vxlog("INFO", "Password free SSH is not set to $dstHost.");
		return CONST_NO;
	}
	else {
		Utils::vxlog("INFO", "Password free SSH is already set to $dstHost.");
		return CONST_YES;
	}
}


sub createKeysForSSH
{
	my $sMyName = ( caller 0 )[3];

	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";

	if ( ! -f $keyFile or ! -f "$keyFile.pub" ) {

		my $cmd = "ssh-keygen -t dsa -f $keyFile -N \"\" -q";

		($ret, $msg) = Utils::runCmd($cmd);
		if ( $ret != CONST_SUCCESS ) {
			$msg = "Could not create keys - [$msg]";
			Utils::vxlog("ERROR", $msg);
		}
	}

	return ($ret, $msg);
}


sub setupPasswdFreeSSH
{
	my $op      = shift;
	my $dstHost = shift;
	my $dstUser = shift;
	my $dstPasswd = shift;

	my $sMyName = ( caller 0 )[3];

	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $href = getVCSPaths();

	my $pubkey = `cat $keyFile.pub`;
	my $xml = "\"<xml><PublicKey>$pubkey</PublicKey></xml>\"";

	$dstPasswd =~ s/'/'"'"'/g;

	my $cmd = $href -> { xprtlc } . " -l https://$dstHost:5634/vcs/admin/setupSSH.pl -d OP=$op -d params=$xml -u \'$dstUser\' -p \'$dstPasswd\'";

	my ( $ret, $msg ) = Utils::runCmd($cmd, { doNotLogCmd => 1 });
	if ( $ret != CONST_SUCCESS ) {
		$msg = "Error occurred while $op to $dstHost - [$msg]";
		Utils::vxlog("ERROR", $msg);
	}

	return ($ret, $msg);
}


sub getBestClusCreateTool
{
	my $matchPattern = shift;

	my $sMyName = ( caller 0 )[3];

	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $cmd = "ls -t " . $matchPattern;

	my ( $ret, $msg ) = Utils::runCmd($cmd);
	if ( $ret != CONST_SUCCESS ) {
		$msg = "Unable to find the installer $matchPattern - [$msg]";
		Utils::vxlog("WARNING", $msg);
		return ($ret, "");
	}

	my @list = split ( /\n/, $msg );
	Utils::vxlog("INFO", "Found installer [" . $list[0] . "]");

	return ( $ret, $list[0] );
}

#--------------------------
# Main code starts here ...
#--------------------------
exportPATHVaribale();

1; # so that perl doesn't complain!



