
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: SettingManager.pm,v 1.6 2012/08/07 11:26:57 stripath Exp $
#

package SettingManager;

use Exporter;
@ISA = ('Exporter');

our @EXPORT = qw(
	readSettingsConfFile
	getSettingDefaultValue
	getSettingValue
	setSettingValue
	getSettingsXML
	 );

use strict;
use warnings;

use Config;
use Data::Dumper;
use POSIX;
use Fcntl qw(:DEFAULT :flock);

use Constants;
use Utils;

use constant MSG_FILE => "appmon.msg";

my $settingsConfFile = "settings.conf";
my $AppSettings      = ();
my $settingsDirPath  = PlatUtils::getHVConfDir();

sub new
{
	my $class = shift;
	my $self = {}; # allocate new hash for object
	bless($self, $class);
	return $self;
}

#------------------------------
# Subroutines to apply Settings 
#------------------------------
sub Apply_App_RestartAttempts
{
	my $self  = shift;
	my $SG	  = shift;
	my $Value = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $msg = "";
	my $ret = CONST_SUCCESS;

	my $vcsobj = new VCS();

	Utils::vxlog("INFO", "Processing SG [$SG]");

	($ret, $msg) = $self -> validateSettingValue ( "App.RestartAttempts", $Value );
	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}

	Utils::vxlog("DEBUG", "Setting RestartLimit for SG $SG to $Value.");

	$vcsobj -> makeConfigWritable();
	($ret, $msg) = $vcsobj -> setAppRestartLimit("", $SG, $Value, "");
	$vcsobj -> dumpAndMakeConfigReadOnly();

	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}

END:
	Utils::vxlog("INFO", "returning from setting App.RestartAttempts");
	return ( $ret, $msg );
}


sub Apply_App_StartStopTimeout
{
	my $self  = shift;
	my $group = shift;
	my $value = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $msg    = "";
	my $dbg	   = "";
	my $ret    = CONST_SUCCESS;
	my $vcsobj = new VCS();
	my $hRef   = ();

	($ret, $msg) = $self -> validateSettingValue ( "App.StartStopTimeout", $value );
	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}

	# Assumption: StartStopTimeout would be set to default value when group
	# is created. Hence, we updated it here.
	my @lValue = ( "-update", UI_TIMEOUT_VAR, $value );

	$hRef -> { SG 	} = $group;
	$hRef -> { sys 	} = CONST_VCS_GLOBAL;
	$hRef -> { attr } = "UserAssoc";
	$hRef -> { value} = \@lValue;

	$vcsobj -> makeConfigWritable();
	($ret, $msg, $dbg) = $vcsobj -> modifyServiceGroup ( $hRef );

	if ( $ret != CONST_SUCCESS ) {
		if ( $dbg =~ /V-16-1-10571/ ) { 
			# This message ID refers to "Key UITimeout not found"
			# Key for StartStopTimeout does not exist. So we will add it.

			@lValue = ( "-add", UI_TIMEOUT_VAR, $value );
			$hRef -> { value } = \@lValue;

			($ret, $msg, $dbg) = $vcsobj -> modifyServiceGroup ( $hRef );

			if ( $ret == CONST_SUCCESS ) {
				goto END;
			}
		}
		$msg = "Unable to set StartStopTimeout value to [$value] for group [$group] - [$dbg]";
		goto END;
	}

END:
	$vcsobj -> dumpAndMakeConfigReadOnly();
	Utils::vxlog("INFO", "returning from setting $sMyName");
	return ( $ret, $msg );
}


sub Apply_App_DisplayName
{
	my $self  = shift;
	my $group = shift;
	my $value = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $msg    = "";
	my $dbg	   = "";
	my $ret    = CONST_SUCCESS;
	my $vcsobj = new VCS();
	my $hRef   = ();

	($ret, $msg) = $self -> validateSettingValue ( "App.DisplayName", $value );
	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}

	# Assumption: DisplayName would be set to some value when group
	# is created. Hence, we updated it here.
	my @lValue = ( "-update", DISPLAY_NAME_VAR, $value );

	$hRef -> { SG 	} = $group;
	$hRef -> { sys 	} = CONST_VCS_GLOBAL;
	$hRef -> { attr } = "UserAssoc";
	$hRef -> { value} = \@lValue;

	$vcsobj -> makeConfigWritable();
	($ret, $msg, $dbg) = $vcsobj -> modifyServiceGroup ( $hRef );

	if ( $ret != CONST_SUCCESS ) {
		if ( $dbg =~ /V-16-1-10571/ ) { 
			# This message ID refers to "Key Name not found"
			# Key for DisplayName does not exist. So we will add it.

			@lValue = ( "-add", DISPLAY_NAME_VAR, $value );
			$hRef -> { value } = \@lValue;

			($ret, $msg, $dbg) = $vcsobj -> modifyServiceGroup ( $hRef );

			if ( $ret == CONST_SUCCESS ) {
				goto END;
			}
		}
		$msg = "Unable to set DisplayName value to [$value] for group [$group] - [$dbg]";
		goto END;
	}

END:
	$vcsobj -> dumpAndMakeConfigReadOnly();
	Utils::vxlog("INFO", "returning from setting $sMyName");
	return ( $ret, $msg );
}


#-----------------
# Misc Subroutines
#-----------------
sub readSettingsConfFile
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $settingsFile = $settingsDirPath . $settingsConfFile;

	my $settingsH = {};

	Utils::vxlog("INFO", "using file [$settingsFile]");
	my $settings = Utils::convertXMLFileToHash ( $settingsFile );

	my $ref = $settings -> { Setting };
	my @SettingList = Utils::convertHashValuesToArray($ref);

	#Utils::vxlog("DEBUG", "Settings read from file: " . Dumper($ref));

	$settingsH -> { Setting } = ();
	$ref = ();

	foreach my $s ( @SettingList ) {
		my $Name = $s -> { Name };
		#Utils::vxlog("INFO", "Reading setting [$Name]");

		my $data = ();	
		$data -> { Name  } = $s -> { Name  };
		$data -> { Desc  } = $s -> { Desc  };
		$data -> { DisplayUnit } = $s -> { DisplayUnit };
		$data -> { DefaultValue  } = $s -> { DefaultValue  };
		$data -> { Constraint } = ();
		$data -> { Constraint } { Type } = $s -> { Constraint } { Type };
		$data -> { Constraint } { Min  } = $s -> { Constraint } { Min  };
		$data -> { Constraint } { Max  } = $s -> { Constraint } { Max  };

		my $vref = "";
		if ( defined $s -> { Constraint } { ValueList } and
			$s -> { Constraint } { ValueList } ne "" and
			defined $s -> { Constraint } { ValueList } { Value } ) {
			$vref = $s -> { Constraint } { ValueList } { Value };
		}

		my @ValueList = Utils::convertHashValuesToArray($vref);

		$data -> { Constraint } { ValueList } = \@ValueList;

		push( @$ref, $data );
	}
#	use strict;
	$settingsH -> { Setting } = $ref;

	return $settingsH;
}


sub getSettingValue
{
	my $self 	= shift;
	my $SG		= shift;
	my $SettingName = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $vcsobj = new VCS();
	my $Value = 0;

	my %assocVal = split(/\t+/, $vcsobj -> getGrpAttrValue ( $SG, "UserAssoc" ));
	Utils::vxlog("DEBUG", "Got values:" . Dumper(%assocVal) );

	if ( $SettingName eq "App.RestartAttempts" ) {
			$Value = $vcsobj -> getGrpAttrValue ( $SG, "OnlineRetryLimit" );
	}
	elsif ( $SettingName eq "App.StartStopTimeout" ) {
		my $UITimeoutVar = UI_TIMEOUT_VAR;
		$Value = $assocVal{ $UITimeoutVar };
	}
	elsif ( $SettingName eq "App.DisplayName" ) {
		my $DisplayNameVar = DISPLAY_NAME_VAR;
		$Value = $assocVal{ $DisplayNameVar };
	}

	if ( not defined $Value or $Value eq "" ) {
		$Value = $self -> getSettingDefaultValue ( $SettingName );
	}

	return $Value;
}


sub getSettingDefaultValue
{
	my $self 	= shift;
	my $SettingName = shift;

	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);
	
	my $settingsH 	= readSettingsConfFile();
	my $ref		= $settingsH->{Setting};
	my @settings  	= Utils::convertHashValuesToArray($ref);

	my $value = "";
	foreach my $s ( @settings ) {
		if ( $s->{Name} eq $SettingName && defined $s->{DefaultValue}) {
			$value = $s->{DefaultValue};
			last;
		}
	}

	return $value;
}


sub isValidSettingValue
{
	my $s	  = shift;
	my $Value = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";

	my $Type = $s -> { Constraint } { Type };
	Utils::vxlog("DEBUG", "Type = [$Type], Value = [$Value]");

	# check the type
	if ( $Type =~ /Integer/i) { 
		if ( $Value !~ /^[0-9]+$/ ) {
			($ret, $msg) = getMessage (MSG_FILE, "SET_INTEGER_INVALID_VALUE", $Value);
			goto END;
		}

		my $Min  = $s -> { Constraint } { Min  };
		my $Max  = $s -> { Constraint } { Max  };

		if ( $Value ne "" and ( $Value < $Min or $Value > $Max ) ) {
			($ret, $msg) = getMessage (MSG_FILE, "SET_INTEGER_NOT_IN_RANGE", $Value, $Min, $Max);
			goto END;
		}
	} elsif ( $Type =~ /Options/i ) {
		my $ValueList = $s -> { Constraint } { ValueList };
		my $found = 0;

		foreach my $PossibleVal ( @$ValueList ) {
			if ($PossibleVal eq $Value) {
				$found = 1;
				last;
			}
		}

		if ( $found == 0 ) {
			($ret, $msg) = getMessage (MSG_FILE, "SET_OPTIONS_INVALID_VALUE", $Value);
			goto END;
		}

	} elsif ( $Type =~ /String/i ) {
		# Any value is good for string.
	} else {
		($ret, $msg) = getMessage (MSG_FILE, "SET_VALUE_INVALID_TYPE", $Type);
		goto END;
	}

END:
	return ($ret, $msg);
}


sub validateSettingValue
{
	my $self 	= shift;
	my $SettingName = shift;
	my $Value	= shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "Saved setting successfully.";

	my $settingsH 	= readSettingsConfFile();
	my $ref		= $settingsH->{Setting};
	my @settings  	= Utils::convertHashValuesToArray($ref);

	foreach my $s ( @settings ) {
		if ( $s->{Name} eq $SettingName ) {
			($ret, $msg) = isValidSettingValue ( $s, $Value );
			if ( $ret != CONST_SUCCESS ) {
				goto END;
			}

			goto END;;
		}
	}

	($ret, $msg) = getMessage ( MSG_FILE, "SET_INVALID_SETTING", $SettingName);

END:
	return ($ret, $msg);
}


sub getSettingsXML
{
	my $self = shift;
	my $group = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $xmlstr = "";
	my $ret    = CONST_SUCCESS;

	my $settingsH 	= readSettingsConfFile();
	my $ref		= $settingsH->{Setting};
	my @settings 	= Utils::convertHashValuesToArray($ref);

	$xmlstr = "\t<Group>\n" .
		"\t\t<Name>" . $group . "</Name>\n";

	foreach my $s ( @settings ) {
		$xmlstr .= "\t\t<Setting>\n";
		$xmlstr .= "\t\t\t<Name>"  . $s -> { Name	}  . "</Name>\n";
		$xmlstr .= "\t\t\t<Value>" . $self -> getSettingValue($group, $s -> {Name})  . "</Value>\n";
		$xmlstr .= "\t\t\t<DisplayUnit>" . $s -> { DisplayUnit } . "</DisplayUnit>\n";
		$xmlstr .= "\t\t\t<DefaultValue>" . $s -> { DefaultValue } . "</DefaultValue>\n";
		$xmlstr .= "\t\t\t<Desc>"  . $s -> { Desc	}  . "</Desc>\n";
		$xmlstr .= "\t\t\t<Constraint>\n";
		$xmlstr .= "\t\t\t\t<Type>" . $s -> { Constraint } { Type } . "</Type>\n"; 
		$xmlstr .= "\t\t\t\t<Min>"  . $s -> { Constraint } { Min }  . "</Min>\n"; 
		$xmlstr .= "\t\t\t\t<Max>"  . $s -> { Constraint } { Max }  . "</Max>\n";

		$ref = $s -> { Constraint } { ValueList };
		$xmlstr .= "\t\t\t\t<ValueList>\n";
		foreach my $PossibleVal ( @$ref ) {
			$xmlstr .= "\t\t\t\t\t<Value>$PossibleVal</Value>\n";
		}
		$xmlstr .= "\t\t\t\t</ValueList>\n";

		$xmlstr .= "\t\t\t</Constraint>\n";
		$xmlstr .= "\t\t</Setting>\n";
	}

	$xmlstr .= "\t</Group>\n";
END:
	return ($ret, $xmlstr);
}


1; # so that perl compiler doesn't complain!
