
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: VCS.pm,v 1.54 2012/08/23 17:09:17 sgupta2 Exp $
#

package VCS;

use strict;
use Data::Dumper;
use Net::Domain qw(hostname);

use Constants;
use Utils;
use PlatUtils;
use PlatVCS;
use MessageUtils;

use constant MSG_FILE => "clusterop.msg";

my $enable_CID_scan = 1;

my ($hagrp, $hares, $haconf, $haclus, $hatype, $hasys, $hastart, $hastop,
	$vcsconfig, $maincf) = ();
my $clusterIDsFile = PlatUtils::getLogDir() . "clusidfile";
my $lltconfigPidFile = PlatUtils::getLockDir() . "lltconfig.pid";

sub new
{
	my $class = shift;

	my $self = {}; # allocate new hash for object
	bless($self, $class);

	# initialize ha cmd paths
	my $href = PlatUtils::getVCSPaths();
	$hagrp	= $href -> { hagrp	 };
	$hares	= $href -> { hares	 };
	$haconf = $href -> { haconf  };
	$hatype = $href -> { hatype  };
	$haclus = $href -> { haclus  };
	$hasys	= $href -> { hasys	 };
	$hastart= $href -> { hastart };
	$hastop = $href -> { hastop	 };

	$vcsconfig = $href -> { ConfigDir };
	$maincf = $href -> { MainCf };

	return $self;
}



sub getClusterDetails
{
	my $self = shift;

	#
	# If this parameter is undefined then, we discover all the details.
	# If it is defined then, only requested discovery details will be returned.
	# Expected Input Hash format is
	#	Cluster			- [Not Used]
	#	ServiceGroup	- Name
	#					- Attributes
	#	Resources		- Name [Not Used]
	#					- Types
	#					- Attributes
	#					- Groups 
	#	ServiceGroupDep	- Groups
	#	ResourcesDep	- [Not Used]
	#	Systems		- Name
	#			- Attributes
	#	
	# If subparameters for each cluster discovery is not passed then by default
	# all the values will be returned.
	# 
	my $clusDiscoveryInputRef = shift;

	if(!defined $clusDiscoveryInputRef)
	{
		$clusDiscoveryInputRef->{Cluster}			= {};
		$clusDiscoveryInputRef->{ServiceGroup}		= {};
		$clusDiscoveryInputRef->{Resources}			= {};
		$clusDiscoveryInputRef->{ServiceGroupDep}	= {};
		$clusDiscoveryInputRef->{ResourcesDep}		= {};
		$clusDiscoveryInputRef->{Systems}		= {};
	}

	my ($depRef, $href, $ret, $msg) = ();
	
	my ($clusRef,  $grpRef, $resRef, $grpDepRef, $resDepRef, $sysRef) = ();

	$ret = CONST_SUCCESS;
	$msg = "";

	if(defined $clusDiscoveryInputRef->{Cluster})
	{

CLUSTER_STEP:
		#TODO: Use specified attributes from Cluster Key 

		my $op = `$haclus -display -localclus`;
		my $CmdRet = $? / 256;
		if ( $CmdRet != 0 ) {
			# V-16-1-10600 means "Cannot connect to VCS engine", which is not an error if VCS is stopped
			if ( $op !~ /V-16-1-10600/ ) {
				$ret = CONST_FAILURE;
			}
			$msg = $op;
			goto END;
		}

		foreach my $line ( split(/\n/, $op) ) {
			if ( $line =~ /^#/ ) {
				next;
			}

			my @cols = split( /\s+/, $line, 2 );
			$clusRef -> { $cols[0] } = $cols[1] || "";
		}
	}

	if(defined $clusDiscoveryInputRef->{Systems})
	{

SYSTEM_STEP:
		my $sysCmd = "$hasys -display ";

		if(defined $clusDiscoveryInputRef->{Systems}{Name} &&
			$#{$clusDiscoveryInputRef->{Systems}{Name}} gt -1)
		{
			$sysCmd .= join " ", @{$clusDiscoveryInputRef->{Systems}{Name}};
		}

		if(defined $clusDiscoveryInputRef->{Systems}{Attributes} &&
			$#{$clusDiscoveryInputRef->{Systems}{Attributes}} gt -1)
		{
			$sysCmd .= " -attribute ".join " ", @{$clusDiscoveryInputRef->{Systems}{Attributes}};
		}

		my $op = `$sysCmd -localclus 2>&1`;
		my $CmdRet = $? / 256;
		chomp $op;

		if ( $CmdRet != 0 ) {
			$msg = $op;
			goto END;
		}

		foreach my $line ( split(/\n/, $op) ) {
			if ( $line =~ /^#/ ) {
				next;
			}

			my @cols = split( /\s+/, $line, 3);
			$sysRef -> { $cols[0] } { $cols[1] } = $cols[2];
		}
	}

	if(defined $clusDiscoveryInputRef->{ServiceGroup})
	{

GROUP_STEP:
		my $grpCmd = "$hagrp -display -localclus ";

		if(defined $clusDiscoveryInputRef->{ServiceGroup}{Name} && 
			$#{$clusDiscoveryInputRef->{ServiceGroup}{Name}} gt -1)
		{		
			$grpCmd = $grpCmd .join " ", @{$clusDiscoveryInputRef->{ServiceGroup}{Name}};
		}

		if(defined $clusDiscoveryInputRef->{ServiceGroup}{Attributes} &&
			$#{$clusDiscoveryInputRef->{ServiceGroup}{Attributes}} gt -1)
		{
			$grpCmd = $grpCmd ." -attribute ".join " ", @{$clusDiscoveryInputRef->{ServiceGroup}{Attributes}};
		}

		my $op = `$grpCmd 2>&1`;
		my $CmdRet = $? / 256;
		chomp $op;

		if ( $CmdRet != 0 ) {
			# 50031 means "No Groups are configured", which is not an error
			if ( $op !~ /50031/ ) {
				$ret = CONST_FAILURE;
			}
			$msg = $op;
			goto END;
		}

		foreach my $line ( split(/\n/, $op) ) {
			if ( $line =~ /^#/ ) {
				next;
			}

			my @cols = split( /\s+/, $line, 4);
			$grpRef -> { $cols[0] } { $cols[1] } { $cols[2] } = $cols[3];

			if ( not defined  $grpRef -> { $cols[0] } { resources } ) {
				$grpRef -> { $cols[0] } { resources } = [];
			}
		}
	}

	if(defined $clusDiscoveryInputRef->{Resources})
	{

RESOURCE_STEP:
		my $resCmd = "$hares -display -localclus";
		if(defined $clusDiscoveryInputRef->{Resources}{Groups} && $#{$clusDiscoveryInputRef->{Resources}{Groups}} gt -1)
		{		
			$resCmd = $resCmd ." -group ".join " ", @{$clusDiscoveryInputRef->{Resources}{Groups}} ;
		}

		if(defined $clusDiscoveryInputRef->{Resources}{Attributes} && $#{$clusDiscoveryInputRef->{Resources}{Attributes}} gt -1)
		{
			$resCmd = $resCmd ." -attribute ".join " ", @{$clusDiscoveryInputRef->{Resources}{Attributes}};
		}

		if(defined $clusDiscoveryInputRef->{Resources}{Types} && $#{$clusDiscoveryInputRef->{Resources}{Types}} gt -1)
		{
			$resCmd = $resCmd ." -type ".join " ", @{$clusDiscoveryInputRef->{Resources}{Types}};
		}

		my $op = `$resCmd 2>&1`;
		my $CmdRet = $? / 256;
		chomp $op;
		if ( $CmdRet != 0 ) {
			# 50030 means "No Resources are configured", which is not an error
			if ( $op !~ /50030/ ) {
				$ret = CONST_FAILURE;
			}
			$msg = $op;
			goto END;
		}

		$resRef = ();
		foreach my $line ( split(/\n/, $op) ) {
			if ( $line =~ /^#/ ) {
				next;
			}

			my @cols = split( /\s+/, $line, 4);
			my $data = ();
			$resRef -> { $cols[0] } { $cols[1] } { $cols[2] } = $cols[3];

			# resource Name is cols[0]!
			# resource name is always global!
			$resRef -> { $cols[0] } { ResourceName } { global } = $cols[0] 
				if ( not defined $resRef -> { $cols[0] } { ResourceName } { global });

			# push the resource in group!
			# cols[0] -> resource name, cols[1] -> attr,
			# cols[2] -> system, cols[3] -> value
			if ( $cols[1] =~ /^group$/i and
				not grep /$cols[0]/, @ {$grpRef -> { $cols[3] } { resources }} ) {
				push (@{$grpRef -> { $cols[3] } { resources }}, $cols[0]);
			}
		}
	}

	if(defined $clusDiscoveryInputRef->{ResourcesDep})
	{

RESOURCE_DEPENDENCY_STEP:
		my $op = `$hares -dep 2>&1`;
		my $CmdRet = $? / 256;
		chomp $op;

		if ( $CmdRet != 0 ) {
			# 50034 means "No Resource dependencies are configured", which is not an error
			if ( $op !~ /50034/ ) {
				$ret = CONST_FAILURE;
			}
			$msg = $op;
			goto GROUP_DEPENDENCY_STEP;
		}

		foreach my $line ( split (/\n/, $op) ) {
			if ( $line =~ /^#/ ) {
				next;
			}

			my $data = ();
			my @cols = split(/\s+/, $line, 3);
			$data -> { Group  } = $cols[0];
			$data -> { Parent } = $cols[1]; 
			$data -> { Child  } = $cols[2]; 
			$data -> { Type   } = "Resource Dependency";

			push (@$resDepRef, $data); 
		}
	}

	if(defined $clusDiscoveryInputRef->{ServiceGroupDep})
	{

GROUP_DEPENDENCY_STEP:
		my $grpDepCmd = "$hagrp -dep ";

		if(defined $clusDiscoveryInputRef->{ServiceGroupDep}{Groups} && $#{$clusDiscoveryInputRef->{ServiceGroupDep}{Groups}} gt -1)
		{		
			$grpDepCmd .= join " ", @{$clusDiscoveryInputRef->{ServiceGroupDep}{Groups}} ;
		}

		my $op = `$grpDepCmd 2>&1`;
		my $CmdRet = $? / 256;
		chomp $op;

		if ( $CmdRet != 0 ) {
			# 50035 means "No Group dependencies are configured", which is not an error
			if ( $op !~ /50035/ ) {
				$ret = CONST_FAILURE;
			}
			$msg = $op;
			goto END;
		}

		foreach my $line ( split (/\n/, $op) ) {
			if ( $line =~ /^#/ ) {
				next;
			}

			my $data = ();
			my @cols = split(/\s+/, $line, 3);
			$data -> { Parent } = $cols[0]; 
			$data -> { Child  } = $cols[1]; 
			$data -> { Type   } = $cols[2];

			push (@$grpDepRef, $data); 
		}
	}

END:
	my $ref = ();

	$ref -> { GroupRef   } = $grpRef;
	$ref -> { ResRef     } = $resRef;
	$ref -> { GrpDepRef  } = $grpDepRef;
	$ref -> { ResDepRef  } = $resDepRef;
	$ref -> { ClusRef    } = $clusRef;
	$ref -> { SysRef     } = $sysRef;

	return ($ret, $msg, $ref);
}



#--------------------------------------------------------
# Create the input file (xml / response file) 
# for the native tool (vcwsilent.exe / CPI)
#--------------------------------------------------------
sub CreateClusterInputFile
{
	my $subName = ( caller 0 )[3];
	Utils::vxlog("INFO", "Subroutine $subName called");

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $hData = ();

	($ret, $hData) = CreateClusterInputFile_plat( @_ );
	if ( $ret != CONST_SUCCESS ) {
		($ret, $msg) = getMessage( MSG_FILE, "NATIVETOOL_INPUT_CREATION_FAILED");
		goto END;
	}

END:
	Utils::vxlog("INFO", "Subroutine $subName end");
	return ($ret, $msg, $hData);
}


#-----------------------------------------
# Launch the native cluster creation tool
#-----------------------------------------
sub LaunchClusterCreateTool
{
	my $subName = ( caller 0 )[3];
	Utils::vxlog("INFO", "Subroutine $subName called");
	return LaunchClusterCreateTool_plat( @_ );
}


#-----------------------------------------
# Unconfigure the cluster through proxy on
# each node of the cluster
#-----------------------------------------
sub UnconfigureClusterViaProxy
{
	my $subName = ( caller 0 )[3];
	Utils::vxlog("INFO", "Subroutine $subName called");
	return UnconfigureClusterViaProxy_plat( @_ );
}


#----------------------------------------
# generate resource display name / label
#----------------------------------------
sub getResourceLabel
{
	my $hRes = shift;

	my $resNameFileFullSpec = PlatUtils::getHVConfDir() . "resName.mf";

	my $resType = $hRes -> { Type } { global };
	my $label = ""; # default value
	my $ref = ();


	no strict;
	do $resNameFileFullSpec;
	$ref = $resLabelConf { $resType };
	use strict;

	if ( defined $ref ) {
		$label = shift @$ref;

		my $ii = 1;
		foreach my $attr ( @$ref ) {
			# TODO: instead of key[0], use the current system / global!
			my @key = keys %{ $hRes -> { $attr }};
 			my $value = $hRes -> { $attr } { $key[0] };
		
			$label =~ s/\%$ii/$value/g;
			$ii++;
		}
	} else {
		$label = $hRes -> { ResourceName } { global };
	}

	return $label;
}


# ---------------------------
# Cluster Details
# ---------------------------
sub getClusterID
{
	my $clusterID;
	my $lltFileName = PlatUtils::getLLTTabFilePath();
	if ( -e $lltFileName) {
		open LLTTAB_FH, "<$lltFileName" or return $clusterID;
	} else {
		Utils::vxlog("ERROR", "$lltFileName files does not exists" );
		return $clusterID;
	}
	
	while( <LLTTAB_FH>)
	{
		if($_ =~ m/^(set-cluster)/i)
		{
			my @clusID = split(/\s+/,$_);
			$clusterID = $clusID[1] if $clusID[1];
			last;
		}
	}
	close LLTTAB_FH;
	
	return $clusterID;
	
}


sub IsSecureCluster
{	
	my $ret = CONST_SUCCESS;
	my $msg = "";
	
	my $bIsSecureClus = "";
	
	my $op = `$haclus -value SecureClus -localclus`;
	my $CmdRet = $? / 256;
	if ( $CmdRet != 0 ) {
		$ret = CONST_FAILURE;
		$msg = $op;
		goto END;
	}
	
	chomp $op;
	$bIsSecureClus = $op;
	
END:
	return ($ret, $msg, $bIsSecureClus);
}


sub getClusterName
{	
	my $hashVCSPaths = PlatUtils::getVCSPaths();
	my $haclus = $hashVCSPaths->{haclus};
	my $haClusListCmd = "$haclus -value ClusterName -localclus";
	return Utils::runCmd("$haClusListCmd 2>&1");
}


sub getRandomClusterIDAndName
{
	my $randClusID = int(rand(65535));
	my $randClusName = "Cluster_$randClusID";
	return ($randClusID,$randClusName);
}


sub getClusterSystemsList
{
	my $retCode 	= CONST_SUCCESS;
	my $errorMsg 	= "";
	my $hashVCSPaths = PlatUtils::getVCSPaths();
	my $hasys = $hashVCSPaths->{hasys};
	my $haSysListCmd = "$hasys -list -localclus";
	my ($cmdRet, $op, $Ret) = Utils::runCmd("$haSysListCmd 2>&1");	
	
	if($cmdRet != CONST_SUCCESS)
	{
	   ( $retCode, $errorMsg ) = getMessage( MSG_FILE, "ERROR_GET_CLUSTER_SYS_LIST");
	   $op = "";
	}
	  
	return ($retCode,$errorMsg,$op);
}


sub getClusterConfigurationStatus
{
	my $clusterConfigured = 0;
	my $hashVCSPaths = PlatUtils::getVCSPaths();
	my $hasys = $hashVCSPaths->{hasys};
	my $haSysStateCmd = "$hasys -state -localclus";
	my ($CmdRet, $op, $Ret) = Utils::runCmd("$haSysStateCmd 2>&1");
	
	if($CmdRet eq CONST_SUCCESS)
	{
		$clusterConfigured = 1;
	}
		
	return $clusterConfigured;
}

# --------------------------------------------------------------------------------------------------------------
# LLT Link Details
#
#link tag device_name systemid_range link_type sap mtu_size [ ip_address broadcast_address | multicast_address ]
# --------------------------------------------------------------------------------------------------------------

sub getLLTLink
{

	my %lltLink = ();
	my $linkLine = shift;
	my $linkID = shift;
	my $isLowPriority = shift;
	$lltLink{LinkID}		=$linkID;
	$lltLink{IsLowPriority} =$isLowPriority; 
	my @linkDetails = split(/ +|\|/,$linkLine);	
	
	shift @linkDetails;
	shift @linkDetails;
	my $mac = shift @linkDetails;
	$lltLink{MAC}=$mac if $mac && $mac ne '-';
	$lltLink{MAC} =~ s/^eth-//;
	shift @linkDetails;
	my $linkType = shift @linkDetails;
	if($linkType)
	{
		$lltLink{LinkType}=$linkType;
		if(lc($linkType) eq 'udp6')
		{
			$lltLink{IPVersion}="IPv6";
		}
		elsif(lc($linkType) eq 'udp')
		{
			$lltLink{IPVersion}="IPv4";
		}
	}
	my $SAPPort = shift @linkDetails;
	$lltLink{SAPPort} = $SAPPort if $SAPPort && $SAPPort ne '-';
	shift @linkDetails;
	my $ipAddress = shift @linkDetails;
	$lltLink{IPAddress} = $ipAddress   if $ipAddress && $ipAddress ne '-';
	my $subnetMask = shift @linkDetails;
	$lltLink{SubnetMask} = $subnetMask  if $subnetMask && $subnetMask ne '-';
		
	return \%lltLink;
}


sub getLLTLinkDetails
{
	my $retCode = CONST_SUCCESS;
	my $errorMsg = "";
	my %lltLinks = ();
	my $linkID = 1;
	my $lltFileName = PlatUtils::getLLTTabFilePath();
	
	if ( -e $lltFileName) {
		open LLTTAB_FH, "<$lltFileName" or die "unable to load.. $lltFileName: $!\n";
	} else {
		Utils::vxlog("ERROR", "$lltFileName files does not exists" );
		( $retCode, $errorMsg ) = getMessage( MSG_FILE, "LLT_FILE_DOESNT_EXISTS");
		return ($retCode,$errorMsg,\%lltLinks);
	}
	
	while(<LLTTAB_FH>) {		
		chomp $_;	
		
		if($_ =~ m/^(link-lowpri)/i)
		{
			my $lltLink = getLLTLink($_,$linkID,1);
			$lltLinks{$linkID++} = $lltLink ;	
		}
		elsif($_ =~ m/^(link)/i)
		{
			my $lltLink = getLLTLink($_,$linkID,0);
			$lltLinks{$linkID++} = $lltLink ;	
		}
	}
	close LLTTAB_FH;
	
	return ($retCode,"",\%lltLinks);
}


sub makeConfigWritable
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $cmd = "$haclus -value ReadOnly -localclus";
	my ($CmdRet, $op) = Utils::runCmd( $cmd );
	chomp $op;
	if ( $CmdRet != CONST_SUCCESS ) {
		return ( $CmdRet, $op );
	}
	if ( $op eq "0" ) {
		return (CONST_SUCCESS, "");
	}

	$cmd = "$haconf -makerw";
	($CmdRet, $op) = Utils::runCmd( $cmd );
	if ( $CmdRet != CONST_SUCCESS ) {
		return ( $CmdRet, $op );
	}

	return ( CONST_SUCCESS, "" );
}


sub dumpAndMakeConfigReadOnly
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $op) = ();
	my $cnt = 3;
	while ($cnt > 0) {
		Utils::runCmd("$haclus -wait DumpingMembership 0");
		($ret, $op) = Utils::runCmd("$haconf -dump -makero");
		if ( $ret == CONST_SUCCESS ) {
			last;
		}
		$cnt --;
		sleep(2);
	}

	if ( $ret != CONST_SUCCESS ) {
		return ($ret, $op);
	}

	return ( CONST_SUCCESS, "" );
}


sub addServiceGroup
{
	my $self = shift;
	my $SGName = shift || "";

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $debug = "";

	$SGName = Utils::strTrim( $SGName );
	if ( $SGName eq "" or $SGName =~ /\s+/ ) {
		($ret, $msg) = getMessage( MSG_FILE, "VCSOP_GRP_ADD_INVALID_SGNAME", $SGName);
		goto END;
	}

	my $cmd = "$hagrp -add $SGName";
	my ($ret, $op) = Utils::runCmd( $cmd );
	if ( $ret != CONST_SUCCESS ) {
		($ret, $msg) = getMessage( MSG_FILE, "VCSOP_GRP_ADD_FAILED", $SGName);
		$debug = "<Debug><![CDATA[$op]]></Debug>";
		goto END;
	}

END:
	return ($ret, $msg, $debug);
}


sub modifyServiceGroup
{
	my $self = shift;
	my $ref = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $debug = "";

	my $SGName = $ref -> { SG };
	my $sys = $ref -> { sys };
	my $attr = $ref -> { attr };
	my $value = $ref -> { value };

	my $val = "";
	my @vals = ();
	if ( "$value" =~ /ARRAY/i ) {
		@vals = @$value;
	} else {
		push(@vals, $value);
	}
	foreach my $v (@vals) {
		$val .= " \"$v\" ";
	}

	my $cmd = "$hagrp -modify $SGName $attr $val";
	if ($sys ne CONST_VCS_GLOBAL) {
		$cmd .= " -sys $sys";
	}

	($ret, $debug) = Utils::runCmd($cmd);
	if ($ret != CONST_SUCCESS) {
		($ret, $msg) = getMessage( MSG_FILE, "VCSOP_GRP_MODIFY_FAILED", $attr, $SGName, $val);
		$debug = "<Debug><![CDATA[$debug]]></Debug>";
	}


EXIT_LABLE:
	return ($ret, $msg, $debug);
}




sub addResource
{
	my $self = shift;
	my $ref = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $debug = "";

	my $resName = $ref -> { Name };
	my $resType = $ref -> { Type };
	my $SGName  = $ref -> { SG };

	my $cmd = "$hares -add $resName $resType $SGName";
	($ret, $debug) = Utils::runCmd($cmd);
	if ($ret != CONST_SUCCESS) {
		($ret, $msg) = getMessage( MSG_FILE, "VCSOP_RES_ADD_FAILED", $resName, $resType, $SGName);
		$debug = "<Debug><![CDATA[$debug]]></Debug>";
	}

EXIT_LABLE:
	return ($ret, $msg, $debug);

}

#-------------------------------------------------------------
# This function checks the existence of a resource
# in vcs configuration based on the attrribute value
# Input param: A hash
#	Hash '$ref' will have the following information
#	1) Type: Use to filter the type of resource to search
#	2) Set of attributres needs to be mached
#	3) Attributre value to look for
#
# Output param: ($retval, $MachedRes)
#	1) $retval will be set to '1' if existing vcs
#	   resource is found. In this case it store resource
#	   name in '$MachedRes'
#	2) $retval will be set to '0' if resource not found
#	   In this case $MachedRes is undefined
#------------------------------------------------------------	
sub isSimilarResourceExist 
{
	my $self = shift;
	my $ref = shift;

	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $debug = "";
	my $vcsobj = new VCS();
	my ($cmdRet, $cmdMsg, $clusRef) = $vcsobj -> getClusterDetails($ref);
	if ( $cmdRet == CONST_FAILURE )  {
		Utils::vxlog("ERROR", "Error occurred while fetching resource details- [$cmdMsg]");
	} else {	
		my $equal = 0;
		my $resRef = $clusRef -> { ResRef };
		foreach my $res ( keys %$resRef ) {
			$equal = 0;
			foreach my $attr (keys %{$ref->{Resources}{AttributesVal}}) {
				foreach my $scope (keys %{$ref->{Resources}{AttributesVal}{$attr}}) {
					if ($resRef->{$res}{$attr}{$scope} eq 
							$ref->{Resources}{AttributesVal}{$attr}{$scope}) {
						$equal = 1;
						next;
					} else {
						$equal = 0;
						# Skip rest of the scope specific attribute matching
						last;
					}
				}
				if ($equal == 0) {
					# Got for next resource as we found mismatch for a attribute
					last;
				}
			}
			if ($equal == 0) {
				# Got for next resource as some of the attribute is not matching
				next;
			} else {
				Utils::vxlog("INFI", "Got existing resource: $res");
				return (1, $res);
			}
		}
	}
	return (0);
}


sub overrideResAttribute
{
	my $self = shift;
	my $ref = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $debug = "";

	my $resName = $ref -> { Name };
	my $attr = $ref -> { attr };

	my $cmd = "$hares -override $resName $attr";
	($ret, $debug) = Utils::runCmd($cmd);
	if ($ret != CONST_SUCCESS) {
		($ret, $msg) = getMessage( MSG_FILE, "VCSOP_RES_OVERRIDE_FAILED", $attr, $resName);
		$debug = "<Debug><![CDATA[$debug]]></Debug>";
	}

EXIT_LABLE:
	return ($ret, $msg, $debug);
}


sub modifyResource
{
	my $self = shift;
	my $ref = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $debug = "";

	my $resName = $ref -> { Name };
	my $sys = $ref -> { sys };
	my $attr = $ref -> { attr };
	my $value = $ref -> { value };

	my $val = "";
	my @vals = ();
	if ( "$value" =~ /ARRAY/i ) {
		@vals = @$value;
	} else {
		push(@vals, $value);
	}
	foreach my $v (@vals) {
		$val .= " \"$v\" ";
	}

	my $cmd = "";
	if ( $sys ne CONST_VCS_GLOBAL ) { 
		$cmd = "$hares -local $resName $attr";
		Utils::runCmd($cmd);
	}

	$cmd = "$hares -modify $resName $attr $val";
	if ($sys ne CONST_VCS_GLOBAL) {
		$cmd .= " -sys $sys";
	}

	($ret, $debug) = Utils::runCmd($cmd);
	if ($ret != CONST_SUCCESS) {
		($ret, $msg) = getMessage( MSG_FILE, "VCSOP_RES_MODIFY_FAILED", $attr, $resName, $val);
		$debug = "<Debug><![CDATA[$debug]]></Debug>";
	}

EXIT_LABLE:
	return ($ret, $msg, $debug);

}


sub createResourceDependency
{
	my $self = shift;
	my $Parent = shift;
	my $Child = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $debug = "";

	my $cmd = "$hares -link $Parent $Child";
	($ret, $debug) = Utils::runCmd($cmd);
	if ($ret != CONST_SUCCESS) {
		($ret, $msg) = getMessage( MSG_FILE, "VCSOP_RESDEP_ADD_FAILED", $Parent, $Child);
		$debug = "<Debug><![CDATA[$debug]]></Debug>";
	}

EXIT_LABLE:
	return ($ret, $msg, $debug);

}


sub createServiceGroupDependency
{
	my $self = shift;
	my $ref = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $debug = "";

	my $Parent = $ref -> { Parent };
	my $Child = $ref -> { Child };
	my $Relationship = $ref -> { Relationship };

	my $cmd = "$hagrp -link $Parent $Child $Relationship";
	($ret, $debug) = Utils::runCmd($cmd);
	if ($ret != CONST_SUCCESS) {
		($ret, $msg) = getMessage( MSG_FILE, "VCSOP_GRPDEP_ADD_FAILED", $Parent, $Child, $Relationship);
		$debug = "<Debug><![CDATA[$debug]]></Debug>";
	}

EXIT_LABLE:
	return ($ret, $msg, $debug);

}

sub getAllResourcesInGroup
{
	my $self = shift;
	my $SGName = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $cmd = "$hares -list Group=$SGName -localclus";
	my ($CmdRet, $op) = Utils::runCmd($cmd);

	my @resources = ();
	if ($CmdRet == CONST_SUCCESS) {
		my @resourcelines = split(/\n/, $op);
		foreach my $line (@resourcelines) {
			my @fields = split(/\s+/, $line);
			if (not grep /^$fields[0]$/, @resources) {
				push(@resources, $fields[0]);
			}
		}
	}

	return \@resources;
}


sub probeAllGroupResources
{
	my $self = shift;
	my $SGName = shift;
	my $sysName = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $debug = "";

	my $resList = $self -> getAllResourcesInGroup($SGName);
	foreach my $res ( @$resList ) {
		my $cmd = "$hares -probe $res -sys $sysName";
		($ret, $debug) = Utils::runCmd($cmd);
		if ($ret != CONST_SUCCESS) {
			($ret, $msg) = getMessage( MSG_FILE, "VCSOP_GRP_PROBERES_FAILED", $res, $sysName);
			$debug = "<Debug><![CDATA[$debug]]></Debug>";
			goto END;
		}
	}

END:
	return ($ret, $msg, $debug);

}


sub deleteAllGroupResources
{
	my $self = shift;
	my $SGName = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $debug = "";

	my $resList = $self -> getAllResourcesInGroup($SGName);
	foreach my $res ( @$resList ) {
		my $cmd = "$hares -delete $res";
		($ret, $debug) = Utils::runCmd($cmd);
		if ($ret != CONST_SUCCESS) {
			($ret, $msg) = getMessage( MSG_FILE, "VCSOP_RES_DEL_FAILED", $res);
			$debug = "<Debug><![CDATA[$debug]]></Debug>";
			goto END;
		}
	}

END:
	return ($ret, $msg, $debug);

}


sub deleteGrpDependency
{
	my $self	= shift;
	my $parentGrp	= shift;
	my $childGrp	= shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";

	my $cmd = "$hagrp -unlink $parentGrp $childGrp";
	my ($CmdRet, $op) = Utils::runCmd( $cmd );
	if ( $CmdRet != CONST_SUCCESS ) {
		$msg = $op;
		$ret = $CmdRet;
		goto EXIT_LABEL;
	}

	$msg = $op;

EXIT_LABEL:
	return ( $ret, $msg );
}

sub deleteServiceGroup
{
	my $self = shift;
	my $SGName = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $debug = "";

	my $cmd = "$hagrp -delete $SGName";
	($ret, $debug) = Utils::runCmd($cmd);
	if ($ret != CONST_SUCCESS) {
		($ret, $msg) = getMessage( MSG_FILE, "VCSOP_GRP_DEL_FAILED", $SGName);
		$debug = "<Debug><![CDATA[$debug]]></Debug>";
	}

EXIT_LABLE:
	return ($ret, $msg, $debug);
}


sub enableAllResourcesInServiceGroup
{
	my $self = shift;
	my $SGName = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $debug = "";

	my $cmd = "$hagrp -enableresources $SGName";
	($ret, $debug) = Utils::runCmd($cmd);
	if ($ret != CONST_SUCCESS) {
		($ret, $msg) = getMessage( MSG_FILE, "VCSOP_GRP_ENABLERES_FAILED", $SGName);
		$debug = "<Debug><![CDATA[$debug]]></Debug>";
	}

EXIT_LABLE:
	return ($ret, $msg, $debug);
}


sub operateSG
{
	my $self = shift;
	my $SGName = shift;
	my $SGOP = shift;
	my $anySys = shift;
	my $extraParams = shift;
	my $sysName = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $vcsnode = $sysName || "";
	my $frozenValue = 0;

#	($ret, $vcsnode) = $self->getVCSNodeName();
#	if ($ret != APPCONTROL_SUCCESS ) {
#		$msg = $vcsnode;
#		goto END;
#	}

	my $op = $SGOP;
	my $checkFrozenVal = 0;

	if ( $SGOP =~ /^freeze$/ ) {
		$checkFrozenVal = 1;
	} elsif ( $SGOP =~ /^unfreeze$/ ) {
		$checkFrozenVal = 0;

		($ret, $frozenValue) = $self -> getGrpAttrValue($SGName, "Frozen");
		if ( $ret != CONST_SUCCESS ) {
			$ret = CONST_FAILURE;
			$msg = "Failed to fetch value for Frozen attribute of group $SGName.";
			goto END;
		}

		if ( $frozenValue == 1 ) {
			$extraParams = " -persistent";
		}
		else {
			$extraParams = "";
		}
	}

	my $addedParams = 0;
	my $cmd = "$hagrp -$op";
	
	# "-force" for flush and "-propagate" for online/offline should come before SG name
	# If any other option is there, need to append it after SG name 
	if ( $SGOP =~ /flush|online|offline/ ) {
		$cmd .= " $extraParams" if ( $extraParams );
		$addedParams = 1;
	}
	
	$cmd .= " $SGName";

	# add extra tags if anything is passed
	# Note: Following options need to be handled after -sys option:
	# 1. -checkpartial for online operation
	# 2. -nopre for switch operation
	if ( ( $addedParams == 0 ) and defined $extraParams ) {
		$cmd .= " $extraParams";
	}

	# add sys tag if available.
	if ( defined $anySys and $anySys == 1 ) {
		$cmd .= " -any";
	}
	else {
		if ( $SGOP =~ /switch/ ) {
			$cmd .= " -to";
		}
		elsif ( $SGOP !~ /freeze/ ) {
			$cmd .= " -sys";
		}
		$cmd .= " $vcsnode";
	}

	# TODO: Add "-localclus" to operations where applicable

	my $cnt  = 3;
	my $done = 0;
	while ( $done == 0 and $cnt > 0 ) {
		Utils::vxlog("INFO", "Checking if cluster is running or not");
		if ( $self -> checkIfVCSRunning () == CONST_YES ) {
			$done = 1;
		} else {
			Utils::vxlog("INFO", "Cluster is not running, sleeping ...");
			sleep(60);
			Utils::vxlog("INFO", "Resuming the operation");
		}
		$cnt --;
	}

	$cnt = 3;
	do {
		($ret, $msg) = Utils::runCmd($cmd);

		if ( $op =~ /freeze|unfreeze/) {
			my $attr = "TFrozen";
			$attr = "Frozen" if ( $extraParams =~ /persistent/ );

			my ($fret, $op) = $self -> getGrpAttrValue($SGName, $attr);
			Utils::vxlog("INFO", "Checking for value [$checkFrozenVal], [$op]");
			if ( $op != $checkFrozenVal ) {
				$ret = CONST_FAILURE;
				sleep(2);
			} else {
				$ret = CONST_SUCCESS;
				goto END;
			}
			$cnt --;
		}
	} while (( $cnt > 0 ) and ( $op =~/freeze|unfreeze/ ));

END:
	return ($ret, $msg);
}


sub getGrpAttrValue
{
	my $self = shift;
	my $grpName = shift;
	my $attr =	shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";

	my $cmd = "$hagrp -value $grpName $attr -localclus";
	my ($CmdRet, $op) = Utils::runCmd( $cmd );
	if ( $CmdRet != CONST_SUCCESS ) {
		$msg = $op;
		$ret = $CmdRet;
		goto EXIT_LABEL;
	}

	$msg = $op;

EXIT_LABEL:
	return ( $ret, $msg );
}

sub isGroupPresent
{
        my $self = shift;
        my $GroupName = shift;

        my $sMyName = ( caller 0 )[3];
        #---------
        # Announce
        #---------
        my $logStr = "Subroutine $sMyName called";
        Utils::vxlog("INFO", $logStr);

        my $cmd = "$hagrp -list";
        my ($CmdRet, $op) = Utils::runCmd($cmd);
        if ($CmdRet != CONST_SUCCESS) {
                return CONST_NO;
        }

        my @groups = split(/\s+/, $op);
        if (grep /^$GroupName$/, @groups) {
                return CONST_YES;
        }

        return CONST_NO;
}

sub setAppRestartLimit
{
	my $self	 = shift;
	my $ApplyLevel	 = shift;
	my $SGName	 = shift;
	my $Value	 = shift;
	my $ApplyOnTypes = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";

	# validations ...
	if ( not defined $ApplyLevel or ($ApplyLevel ne "" and $ApplyLevel !~ /resource/i and $ApplyLevel !~ /serviceGroup/i) ) {
		($ret, $msg) = getMessage ( MSG_FILE, "SET_RESTARTATTEMPTS_INVALID_APPLYLEVEL");
		goto END;
	}

	if ( $ApplyLevel eq "" ) {
		$ApplyLevel = "serviceGroup";
	}

	if ( not defined $SGName ) {
		($ret, $msg) = getMessage ( MSG_FILE, "SETINGS_INVALID_SG_NAME");
		goto END;
	}


	# Initialize the values ...
	my $FinalValue = 0;
	if ($ApplyLevel =~ /serviceGroup/) {
		$FinalValue = $Value;
	}

	my $cmd = "$hagrp -modify $SGName OnlineRetryLimit $FinalValue";
	($ret, $msg) = Utils::runCmd($cmd);
	if ($ret != CONST_SUCCESS) {
		goto END;
	}

END:
	return ($ret, $msg);
}


sub checkIfVCSRunning
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $cmd = "$haclus -state -localclus";
	my ($CmdRet, $op) = Utils::runCmd($cmd);
	if ( $CmdRet != CONST_SUCCESS ) {
		return CONST_NO;
	}
	CONST_YES;
}


sub waitForVCS
{
	my $self = shift;
	my $timeout = shift;
	my $nodeName = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_YES;

	if ( not defined $nodeName or $nodeName eq "" ) {
		$nodeName = hostname();
	}

	while (1) {
		my $cmd = "$hasys -wait $nodeName SysState RUNNING -time $timeout";
		my ($CmdRet, $op) = Utils::runCmd($cmd);
		if ( $CmdRet != CONST_SUCCESS ) {

			if ( ( $op =~ /V-16-1-10600/ or $op =~ /V-16-1-53007/ ) and $timeout > 0 ) {
			# V-16-1-10600 Cannot connect to VCS engine
			# V-16-1-53007 Error returned from engine: HAD on this node not accepting clients - authserver might take some time to come up in secure cluster
			# For both the above errors wait for 1 sec and try again
				sleep(1);
				$timeout--;
				next;
			}

			$ret = CONST_NO;
			last;
		}
		else {
			$ret = CONST_YES;
			last;
		}
	}

	return $ret;
}


sub stopVCS
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $msg = "";

	my $cmd = "$hastop -all";
	my ($CmdRet, $op) = Utils::runCmd($cmd);
	if ( $CmdRet != CONST_SUCCESS ) {
		return ($CmdRet, $op);
	}

	return ($CmdRet, $msg);
}

sub getServiceGroups
{
	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);
	my ($cmdRet, $msg, $op);

	my $cmd = "$hagrp -list -localclus";
	($cmdRet, $op) = Utils::runCmd($cmd);
	if ( $cmdRet != CONST_SUCCESS ) {
		$msg = $op;
		Utils::vxlog("ERROR", "Failed to execute hagrp -list Error message=".$op);
		goto END;
	}

	my $serviceGroupHash;

	foreach my $line ( split(/\n/, $op) ) {
		if ( $line =~ /^#/ ) {
			next;
		}

		my @cols = split( /\s+/, $line, 2 );
		$serviceGroupHash -> { $cols[0] } = $cols[1] || "";
	}

END:
	return ($cmdRet,$msg, keys %$serviceGroupHash);
}


sub getResources
{
	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);
	
	shift;
	my $sgNameStr		 = shift;
	my $resourceTypeStr  = shift;
	my ($cmdRet, $msg, $op);

	my $cmd = "$hares -list -localclus";	
	$cmd .= " Group=" . $sgNameStr if defined $sgNameStr;
	$cmd .= " Type=" . $resourceTypeStr if defined $resourceTypeStr;

	($cmdRet, $op) = Utils::runCmd($cmd);
	if ( $cmdRet != CONST_SUCCESS ) {
		$msg = $op;
		Utils::vxlog("ERROR", "Failed to execute hares -list Error message=" . $op);
		goto END;
	}

	my $resourcesHash;

	foreach my $line ( split(/\n/, $op) ) {
		if ( $line =~ /^#/ ) {
			next;
		}

		my @cols = split( /\s+/, $line, 2 );
		$resourcesHash -> { $cols[0] } = $cols[1] || "";
	}

END:
	return ($cmdRet,$msg, keys %$resourcesHash);
}


sub isResTypeAvailable
{
	my $self = shift;
	my $ResType = shift;
	
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);
	if ( !defined $ResType ) {
		return CONST_FAILURE;
	}
	
	my $cmd = "$hatype -list";
	my ($CmdRet, $op) = Utils::runCmd( $cmd );
	if ( $CmdRet != CONST_SUCCESS ) {
		return $CmdRet;
	}
	
	my @Agents = split(/\n/, $op);
	
	if (grep (/^$ResType$/, @Agents)) {
		return CONST_YES;
	}
	
	return CONST_NO;
}


sub createLLTTabForLLTConfig
{
	my ($isUDP, @aMACs) = @_;
# TODO: Check how UDP links are represented in llttab and handle it

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $sLLTFileName = PlatUtils::getLLTTabFilePath();
	my $i = 1;

	if ( !open ( LLTFILE , "> $sLLTFileName") ) {
		Utils::vxlog("ERROR", "Could not open file for writing $sLLTFileName [$!]");
		return CONST_FAILURE;
	}

	my $hOSInfo = Utils::getOSInfo();
	# set-node is required for windows lltconfig
	print LLTFILE "set-node " . $hOSInfo -> { COMPUTERNAME } . "\n\n" if ( $^O =~ m/win/i );

	foreach my $MAC ( @aMACs ) {
=comment
linux:
link eth1 eth-00:50:56:97:58:63 - ether - -
link eth0 eth-00:50:56:97:58:62 - ether - -

windows:
link Adapter0 00:50:56:96:00:16 - ether - -
link Adapter1 00:50:56:96:00:17 - ether - -
=cut

		if ($^O =~ /lin/i ) {
			# Linux
			print LLTFILE "link eth$i eth-$MAC - ether - -", "\n" ;
		} else {
			# Windows
			print LLTFILE "link Adapter$i $MAC - ether - -", "\n" ;
		}
		$i++;
	}

	return CONST_SUCCESS;
}

sub createLLTHostsForLLTConfig
{

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $sLLTFileName = PlatUtils::getLLTHostsFilePath();
	my $i = 1;

	if ( !open ( LLTFILE , "> $sLLTFileName") ) {
		Utils::vxlog("ERROR", "Could not open file for writing $sLLTFileName [$!]");
		return CONST_FAILURE;
	}

	my $hOSInfo = Utils::getOSInfo();
	# set-node is required for windows lltconfig
	print LLTFILE "0 " . $hOSInfo -> { COMPUTERNAME } . "\n\n" if ( $^O =~ m/win/i );

	return CONST_SUCCESS;
}


sub fetchMACFromXML
{
	my ($tree, $onlyClusIDRequired) = @_;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my @aMACs = ();
	my $isUDP = 0;
	my $clusterID = 0;

	my $discoverData = $tree -> { DiscoveryData };
	if ( $discoverData && $discoverData -> { Cluster } ) {
		my $cluster = $discoverData -> { Cluster };

		my $operation = $cluster -> { Operation };
		my $clusterName = $cluster -> { ClusterName };
		$clusterID = $cluster -> { ClusterID };
		my $lltType = $cluster -> { LLTType };

		if ( ( not defined $onlyClusIDRequired or $onlyClusIDRequired == 0 ) 
				and ( $operation =~ /new/i ) ) {

			if ( $lltType =~ /^udp/i ) {
				$isUDP = 1;
			}

			if ( $cluster -> { NodeList } && $cluster -> { NodeList } -> { Node } ) {

				my @arrNode = Utils::convertHashValuesToArray($cluster -> {NodeList} -> {Node});

				foreach my $node ( @arrNode ) {
					my $nodeName = $node -> { Name };
					my $isLocalhost = $node -> { IsLocalhost };

					if ( $isLocalhost and $isLocalhost == 1 ) {
						if ( $node -> { LLTLink } ) {
							my @arrLinks = Utils::convertHashValuesToArray( $node -> {LLTLink} );

							foreach my $link ( @arrLinks ) {
								if ( $lltType =~ /^udp/i ) {
								# TODO: write a function to create LLTtab file for UDP
								}
								else {
									push ( @aMACs, $link -> { MAC } );
								}
							}
						}
						else {
							# TODO: Error if no LLT link specified
						}
					}
				}
			}
		}
	}

	return ( $clusterID, $isUDP, @aMACs );
}


sub doesClusIDExist 
{
	my $clusIDToCheck = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	open (CLUSIDFILE, "<", $clusterIDsFile) or do {
		Utils::vxlog("ERROR", "Cannot read from file $clusterIDsFile ($!)");
		return CONST_NO; # report failure
	};

	my @existingClusIDs = <CLUSIDFILE>;
	close (CLUSIDFILE);

	if ( grep ( /^$clusIDToCheck$/, @existingClusIDs ) ) {
		return CONST_YES;
	}
	else {
		return CONST_NO;
	}
}


sub initiateCIDScan
{
	my $self = shift;
	my $tree = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called........";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	if ( $enable_CID_scan != 0 ) {
		my ( $clusID, $isUDP, @aMACs ) = fetchMACFromXML ( $tree, 0 );

		$ret = createLLTTabForLLTConfig( $isUDP, @aMACs );
		if ( $ret == CONST_FAILURE ) {
			$msg = "Error in creating llttab file";
			goto END;
		}

		if ( $^O =~ m/win/i )
		{
			$ret = createLLTHostsForLLTConfig( );
			if ( $ret == CONST_FAILURE ) {
				$msg = "Error in creating llthosts file";
				goto END;
			}
		}

		my $href = PlatUtils::getVCSPaths();
		
		Utils::vxlog("INFO", "Invoking readAndValidateSavedPID=".$lltconfigPidFile);
		# Check whether any previous script is running before forking another script
		if ( ( -e $lltconfigPidFile ) and ( my $pid = Utils::readAndValidateSavedPID($lltconfigPidFile) ) ) {
			if ( $pid ) {
				$ret = Utils::killPID( $pid );
				if ( $ret != CONST_SUCCESS ) {
					$msg = "Error occurred while aborting previous command";
					goto END;
				}
			}
			unlink ( $lltconfigPidFile );
		}
		my ( $ret ) = PlatUtils::runCmdAsync( "perl", "\"".$href -> { AdminDir }."fetchClusterIDs.pl\"", $href -> { SFMHPerl });

		if ( $ret != CONST_SUCCESS ) {
			$msg = "Error in fetching existing cluster IDs.";
			goto END;
		}

		Utils::vxlog("INFO", "fetchClusterIDs.pl has been invoked");
	}

END:
	return ( $ret, $msg, $xml );
}


sub waitForCIDScan
{
	my $self = shift;
	my $tree = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	my $randomID = int ( rand ( MAX_CLUSTER_ID ) );

	$xml = "<xml>\n";

	if ( $enable_CID_scan != 0 ) {
		while ( 1 ) {
			if ( ( -e $lltconfigPidFile ) and ( my $pid = Utils::readAndValidateSavedPID($lltconfigPidFile) ) ) {
				Utils::vxlog("DEBUG", "lltconfig is still running. Will wait for 1 sec.");
				sleep ( 1 );
			}
			else {
				Utils::vxlog("DEBUG", "lltconfig has completed. Will return with random valid cluster ID");
				last;
			}
		}

		open (CLUSIDFILE, "<", $clusterIDsFile) or do {
			Utils::vxlog("ERROR", "Cannot read from file $clusterIDsFile ($!)");
			$ret = CONST_FAILURE; # report failure
			$msg = "Unable to open the file $clusterIDsFile.";
			goto END;
		};

		$xml .= "\t<DiscoveredClusterIDs>\n";
		while ( <CLUSIDFILE> ) {
			chomp;
			$xml .= "\t\t<ClusterID>$_</ClusterID>\n";
		}
		close (CLUSIDFILE);
		$xml .= "\t</DiscoveredClusterIDs>\n";

		while ( 1 ) {
			if ( doesClusIDExist ($randomID) == CONST_YES ) {
				Utils::vxlog( "DEBUG", "Cluster ID [$randomID] is in use.");
			}
			else {
				Utils::vxlog( "INFO", "Available Cluster ID is: [$randomID]");
				$xml .= "\t<AvailableClusterIDs>\n\t\t<ClusterID>" .
					$randomID . "</ClusterID>\n\t</AvailableClusterIDs>\n";
				last;
			}

			$randomID = int ( rand ( MAX_CLUSTER_ID ) );
		}
	}
	else {
		$xml .= "\t<AvailableClusterIDs>\n\t\t<ClusterID>" .
			$randomID . "</ClusterID>\n\t</AvailableClusterIDs>\n";
	}

	$xml .= "</xml>";

END:
	return ($ret, $msg, $xml);
}


sub sendFullSnap
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";

	my $href = PlatUtils::getVCSPaths();
	my $notifySinkCmd = $href -> { notifysink } . " -f";
	
	my ($cmdRet, $cmdMsg) = Utils::runCmd($notifySinkCmd);
	if ( $cmdRet != CONST_SUCCESS ) {
		Utils::vxlog ( "WARNING", "Failed to send full snap [$cmdMsg]" );
		$ret = CONST_FAILURE;
	}

	return ($ret, $msg);
}

sub isResourceOfTypeConfigured
{
	my $type = shift;

	my $op = `$hatype -resources $type`;
	if ( $op =~ /\S/ ) {
		Utils::vxlog ( "INFO", "resource of [$type] found" );
		return 1; 
	} else {
		Utils::vxlog ( "INFO", "resource of [$type] not found" );
		return 0; 
	}
}



sub startLLT
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";

	($ret, $msg) = StartLLT_plat ();

	return ($ret, $msg);
}


sub stopLLT
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";

	($ret, $msg) = StopLLT_plat ();

	return ($ret, $msg);
}

sub getAllResources
{
	my $self = shift;

        my $sMyName = ( caller 0 )[3];
        #---------
        # Announce
        #---------
        my $logStr = "Subroutine $sMyName called";
        Utils::vxlog("INFO", $logStr);

        my $cmd = "$hares -list -localclus";
        my ($CmdRet, $op) = Utils::runCmd($cmd);

        my %resources = {};
        if ($CmdRet == CONST_SUCCESS) {
                %resources = split(/\s+/, $op);
        }

        return \%resources;
}

sub getAllGroups
{
	my $self = shift;

        my $sMyName = ( caller 0 )[3];
        #---------
        # Announce
        #---------
        my $logStr = "Subroutine $sMyName called";
        Utils::vxlog("INFO", $logStr);

        my $cmd = "$hagrp -list -localclus";
        my ($CmdRet, $op) = Utils::runCmd($cmd);

        my %groups = {};
        if ($CmdRet == CONST_SUCCESS) {
                %groups = split (/\s+/, $op);
        }

        return \%groups;
}

1; # so that perl doesn't complain!









