#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

# CPIC::verify_systems has 6 steps
#    STEP                         RUN
# 1. installed_prod_sys, prod_version_sys   always
# 2. prod_prerequisite_sys        install_check_task
# 3. rel_platvers_sys             always
# 4. fsspace_sys                  install_check_task && !nospacecheck
# 5. vxlicrep_sys                 install_check_task
# 6. prod_precheck_sys            always, prod sub has appropriate task filters

# we can define this per-check uniquely, but currently is the same for the 3 optional tasks
sub install_check_task { return Cfg::opt(qw(precheck install patchupgrade upgrade)); }

sub systems {
    my $cpic=shift;
    my ($cfg,$default_sys_names,$edr,$rel,$web,$verify_passed,$skip_checks);

    $cfg = Obj::cfg();
    $edr=Obj::edr();
    $web=Obj::web();
    $rel=$cpic->rel();
    $cpic->discover_common_padv() if ($cpic->{padv} eq 'Common');
    # always requires input
    Cfg::unset_opt('silent');
    while (!$verify_passed) {
        $default_sys_names = $cpic->prod_default_systemnames();
        if (!$cfg->{systems} && Obj::webui()) {
            my $msg = Msg::new("Enter system names separated by spaces and click Next", 30, 1326);
            $web->web_script_form('selectsystems',$msg);
        }
        return '' if (Cfg::opt('back'));
        $edr->cli_select_systems($default_sys_names);
        $edr->{savelog}=1; # from this point on log locations are displayed
        $cpic->{systems}=$edr->init_sys_objects();
        $cpic->check_memory_swap_for_localsys() if (!EDR::local_windows());
        $web->web_script_form('precheck') if (Obj::webui());
        $cpic->cli_set_title();

        $cpic->verify_systems_steps();
        Cfg::opt('serial') ? $edr->verify_systems_serial() :
            $edr->verify_systems_threaded($cpic->{threadlimit});

        if ((!EDR::local_windows()) && (!Cfg::opt("responsefile")) &&
             (EDR::com_deny_sys())) {
                 $edr->{com_setup}=$edr->ask_com_setup();
                 next if ($edr->{com_setup}>0);
                 $skip_checks=1 if ($edr->{com_setup}<0);
        }

        # create padv objects for main thread for each system
        $cpic->create_system_padv_objects();

        if ((!$skip_checks) && (EDR::continue_checks())) {
            $cpic->create_system_cpip_objects();
            Cfg::opt('serial') ? $cpic->verify_systems_serial() :
                                 $cpic->verify_systems_threaded();
            $cpic->verify_all_systems();
        }
        $verify_passed=$cpic->verify_system_checks;
        delete $cfg->{systems} unless ($verify_passed);
        # verify_system_checks may proceed with successful systems only
        $cpic->{systems}=$edr->{systems};
        $cpic->cli_set_title();
    }

    $cpic->precheck_report() if (Cfg::opt('precheck'));
    $cpic->handle_precheck_issues();
    $rel->read_licenses() if ((Cfg::opt(qw(install configure))) &&
                              (!$cpic->{eebi}));
    return;
}

# broken out to override, if necessary
sub verify_system_checks {
    my $cpic=shift;
    my $edr=Obj::edr();
    return 1 if (Cfg::opt(qw(rwe precheck)));
    return 1 if ($edr->verify_systems_errors());
    return '' if (Obj::webui());

    $cpic->rel->completion();
    $edr->exit_noexitfile();
    return;
}

sub create_system_padv_objects {
    my $cpic=shift;
    my ($padv);

    # create padv objects for main thread for each system
    for my $sys (@{$cpic->{systems}}) {
        for my $padv($sys->{padv}, $cpic->{padvisa}{$sys->{padv}}) {
            "Padv\::$padv"->new() if ($padv && (!Obj::created('Padv', $padv)));
        }
    }
    return;
}

sub create_system_cpip_objects {
    my $cpic=shift;
    my ($padv,$rel);

    # create CPIP objects for main thread for each system
    for my $sys (@{$cpic->{systems}}) {
        next if ($sys->{stop_checks});
        $padv=$cpic->{padvisa}{$sys->{padv}};
        if ($padv && !Obj::created('Rel', $cpic->{release}, $padv)) {
            $rel=$cpic->create_cpip_objects($padv);
            $cpic->verify_media($rel);
        }
        $padv=$sys->{padv};
        if ($padv && !Obj::created('Rel', $cpic->{release}, $padv)) {
            $rel=$cpic->create_cpip_objects($padv);
        }
    }
    return;
}

sub verify_systems_serial {
    my $cpic=shift;
    my($failmsg,$passmsg,$msg,$rtn,$pc_steps,$pdfrs,$rel,$stage);

    $failmsg=Msg::new("Failed", 30, 1203);
    $passmsg=Msg::new("Done", 30, 1204);

    $stage = Msg::new("Verifying systems", 30, 1327);
    $rel=$cpic->rel();
    for my $sys (@{$cpic->{systems}}) {
        next if (($sys->{stop_checks}) || ($sys->{wmi}));
        $msg=Msg::new("Checking installed product on $sys->{sys}", 30, 1328, "$sys->{sys}");
        $stage->display_left($msg);
        $cpic->installed_prod_sys($sys);
        $rtn=$cpic->prod_version_sys($sys);
        $msg=$rtn ? $passmsg : $failmsg;
        $stage->display_right($msg);
    }
    $rel->upgrade_prod_mix_check() if ((EDR::no_stop_checks()) &&
             (((install_check_task()) || (Cfg::opt('uninstall')))));

    for my $sys (@{$cpic->{systems}}) {
        next if (($sys->{stop_checks}) || ($sys->{wmi}));
        if (install_check_task()) {
            $pdfrs=Msg::get('pdfrs');
            $msg=Msg::new("Checking prerequisite $pdfrs and patches on $sys->{sys}", 30, 1329, "$pdfrs", "$sys->{sys}");
            $stage->display_left($msg);
            $rtn=$cpic->prod_prerequisite_sys($sys);
            $msg=$rtn ? $passmsg : $failmsg;
            $stage->display_right($msg);
        }

        next if ($sys->{stop_checks});
        $msg=Msg::new("Checking platform version level on $sys->{sys}", 30, 1330, "$sys->{sys}");
        $stage->display_left($msg);
        $rtn=$cpic->rel_platvers_sys($sys);
        $msg=$rtn ? $passmsg : $failmsg;
        $stage->display_right($msg);

        next if ($sys->{stop_checks});
        if ((install_check_task() || $rel->rollback_check_task()) && (!Cfg::opt("nospacecheck"))) {
            $msg=Msg::new("Checking file system free space on $sys->{sys}", 30, 1331, "$sys->{sys}");
            $stage->display_left($msg);
            $rtn=$cpic->fsspace_sys($sys);
            $msg=$rtn ? $passmsg : $failmsg;
            $stage->display_right($msg);
        }

        next if ($sys->{stop_checks});
        if (install_check_task()) {
            $msg=Msg::new("Checking product licensing on $sys->{sys}", 30, 1332, "$sys->{sys}");
            $stage->display_left($msg);
            $rtn=$cpic->rel_read_licenses_sys($sys);
            $msg=$rtn ? $passmsg : $failmsg;
            $stage->display_right($msg);
        }

        next if (EDR::stop_checks());
        $msg=Msg::new("Performing product prechecks on $sys->{sys}", 30, 1333, "$sys->{sys}");
        $stage->display_left($msg);
        $pc_steps=$cpic->prod_precheck_steps();
        if ($pc_steps != 0) {
            $msg=Msg::new("Details below", 30, 1334);
            $stage->display_right($msg);
        }
        $cpic->prod_precheck_task();
        $rtn=$cpic->prod_precheck_sys($sys);
        $msg=$rtn ? $passmsg : $failmsg;
        $stage->display_right($msg) if ($pc_steps == 0);
    }
    return '';
}

sub verify_systems_threaded {
    my $cpic=shift;
    my ($failmsg,$passmsg,$msg,$prod,$thr,$rel,$stage,$pc_steps,$pdfrs,$rtns,$rtn);

    $failmsg=Msg::new("Failed", 30, 1203);
    $passmsg=Msg::new("Done", 30, 1204);

    $rel=$cpic->rel();
    $thr=Obj::thr();
    $thr->mq_create_threads($cpic->nsystems, $cpic->{threadlimit});

    $stage = Msg::new("Verifying systems", 30, 1327);

    if (EDR::continue_checks()) {
        $msg=Msg::new("Checking installed product", 30, 1335);
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'installed_prod_sys');
        $thr->mq_wait_for_completion();
        $thr->mq_read_rtns();

        # one bad system will break this check
        if ((EDR::no_stop_checks()) &&
            ((install_check_task()) || (Cfg::opt('uninstall')))) {
            if ($rel->upgrade_prod_mix_check()) {
                $thr->mq_join_threads();
                $thr->mq_create_threads($cpic->nsystems,
                                        $cpic->{threadlimit});
            }
        }

        $thr->mq_add_sub_allsys('CPIC', 'prod_version_sys');
        $thr->mq_wait_for_completion();
        $rtns=$thr->mq_read_rtns();
        $rtn=$thr->mq_check_rtns($rtns);
        $msg=$rtn ? $passmsg : $failmsg;
        $stage->display_right($msg);
    }

    if ((EDR::continue_checks()) && (install_check_task())) {
        $pdfrs=Msg::get('pdfrs');
        $msg=Msg::new("Checking prerequisite patches and $pdfrs", 30, 1336, "$pdfrs");
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'prod_prerequisite_sys');
        $thr->mq_wait_for_completion();
        $rtns=$thr->mq_read_rtns();
        $rtn=$thr->mq_check_rtns($rtns);
        $msg=$rtn ? $passmsg : $failmsg;
        $stage->display_right($msg);
    }

    if (EDR::continue_checks()) {
        $msg=Msg::new("Checking platform version", 30, 1337);
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'rel_platvers_sys');
        $thr->mq_wait_for_completion();
        $rtns=$thr->mq_read_rtns();
        $rtn=$thr->mq_check_rtns($rtns);
        $msg=$rtn ? $passmsg : $failmsg;
        $stage->display_right($msg);
    }

    if ((EDR::continue_checks()) && (install_check_task()||$rel->rollback_check_task()) && (!Cfg::opt('nospacecheck'))) {
        $msg=Msg::new("Checking file system free space", 30, 1338);
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'fsspace_sys');
        $thr->mq_wait_for_completion();
        $rtns=$thr->mq_read_rtns();
        $rtn=$thr->mq_check_rtns($rtns);
        $msg=$rtn ? $passmsg : $failmsg;
        $stage->display_right($msg);
    }

    if ((EDR::continue_checks()) && (install_check_task())) {
        if ($rel->can("read_licenses_sys")) {
            $msg=Msg::new("Checking product licensing", 30, 1339);
            $stage->display_left($msg);
            $thr->mq_add_sub_allsys('CPIC', 'rel_read_licenses_sys');
            $thr->mq_wait_for_completion();
            $rtns=$thr->mq_read_rtns();
            $rtn=$thr->mq_check_rtns($rtns);
            $msg=$rtn ? $passmsg : $failmsg;
            $stage->display_right($msg);
        }
    }

    # cluster checks reference all systems, so all must be good at this point
    if (((EDR::offer_passed_systems()) && (EDR::continue_checks())) ||
         (EDR::no_stop_checks())) {
        $msg=Msg::new("Performing product prechecks", 30, 1340);
        $stage->display_left($msg);
        $pc_steps=$cpic->prod_precheck_steps();
        if ($pc_steps != 0) {
            $msg=Msg::new("Details below", 30, 1334);
            $stage->display_right($msg);
        }
        $cpic->prod_precheck_task();
        $thr->sync_setq();
        $thr->mq_add_sub_allsys('CPIC', 'prod_precheck_sys');
        $thr->mq_wait_for_completion();
        $rtns=$thr->mq_read_rtns();
        $rtn=$thr->mq_check_rtns($rtns);
        $msg=$rtn ? $passmsg : $failmsg;
        $stage->display_right($msg) if ($pc_steps==0);
    }

    $thr->mq_join_threads();
    return '';
}

sub verify_systems_steps {
    my $cpic=shift;
    my ($edr,$steps);
    $cpic->{falsefailures}=0;
    $edr=Obj::edr();
    # There are 2 checks in EDR::verify_systems and 3 required checks in CPIC::verify_systems
    $steps=5;
    # 3 CPIC::verify_systems checks are optional
    # check_fsspace can be skipped with -nospacecheck
    $steps+=2 if (install_check_task());
    $steps++ if ((install_check_task()) && (!Cfg::opt('nospacecheck')));

    $steps *= $cpic->nsystems if (Cfg::opt('serial'));
    $edr->set_progress_steps($steps);
    return $steps;
}

sub check_memory_swap_for_localsys {
    my $cpic=shift;
    my ($localsys,$sys1,$edr,$msg,$nsystems,$nthreads);
    my ($memory_size,$small_cluster_memory,$large_cluster_memory);
    my ($swap_size,$minimum_swap_size);

    # The RAM/swap requirements for successfully installation based on the number of install nodes:
    #
    #  1. If the number install nodes is greater than 8 nodes,  it is recommended to have at least 2G RAM on the install driver node, otherwise 1G is recommended.
    #  2. If the number install nodes is greater than 8 nodes,  it is required to have minimal 1G of free swap space on the install driver node, otherwise, (<num_of_nodes> + 1) * 128 M of free swap space is required.
    #
    # For more informations about the requirements, you can refer to the following incidents:
    #  1846892:    minimal free swap space requirements
    #  2107764:    recommended RAM requirements for large cluster installation.

    $localsys=Obj::localsys();
    $edr=Obj::edr();
    $nsystems=$cpic->nsystems();
    $sys1=$cpic->{systems}->[0];

    if ($nsystems>4 && !Cfg::opt('serial')) {
        $memory_size=$localsys->padv->memory_size_sys($localsys);
        if ($memory_size) {
            # Convert the size based on MegaByte.
            if ($memory_size=~/\s*(\d+)\s*[kK]/mx) {
                $memory_size=int($1/1024);
            } elsif ($memory_size=~/\s*(\d+)\s*[mM]/mx) {
                $memory_size=$1;
            } elsif ($memory_size=~/\s*(\d+)\s*[gG]/mx) {
                $memory_size=$1*1024;
            }

            # large cluster
            $large_cluster_memory=$edr->{large_cluster_memory} || 2048;
            if ($memory_size < $large_cluster_memory) {
                $msg = Msg::new("The available memory on the local system '$localsys->{sys}' is $memory_size MB. It is recommended to have $large_cluster_memory MB of memory, or use the '-serial' option to run the $cpic->{script} program.", 30, 1341, "$localsys->{sys}", "$memory_size", "$large_cluster_memory", "$cpic->{script}");
                $sys1->push_warning($msg);
            }

            $small_cluster_memory=$edr->{small_cluster_memory} || 1280;
            if ($memory_size < $small_cluster_memory) {
                # Set threadlimit to 4
                $cpic->{threadlimit}=4;
            }
        }
    }

    $nthreads=$cpic->{threadlimit};
    if ($nsystems<$nthreads) {
        $nthreads=$nsystems+1;
    }

    # Set minimum swap size as $nthreads * 128M.
    $minimum_swap_size=$nthreads * 128;

    $swap_size=$localsys->padv->swap_size_sys($localsys);
    if ($swap_size) {
        # Convert the size based on MegaByte.
        if ($swap_size=~/\s*(\d+)\s*[kK]/mx) {
            $swap_size=int($1/1024);
        } elsif ($swap_size=~/\s*(\d+)\s*[mM]/mx) {
            $swap_size=$1;
        } elsif ($swap_size=~/\s*(\d+)\s*[gG]/mx) {
            $swap_size=$1*1024;
        }
        if ($swap_size < $minimum_swap_size) {
            $msg = Msg::new("The free swap space on local system '$localsys->{sys}' is $swap_size MB. It is recommended to setup minimal $minimum_swap_size MB of free swap space to successfully run the $cpic->{script} program.", 30, 1342, "$localsys->{sys}", "$swap_size", "$minimum_swap_size", "$cpic->{script}");
            $msg->die;
        }
    }

    return '';
}

sub verify_list {
    my ($cpic,$cpici,$sysi) = @_;
    my ($index,$sys,@list);
    $sysi||=$cpici;
    for my $index (@{$cpic->{$cpici}}) {
        for my $sys (@{$cpic->{systems}}) {
            if (EDRu::inarr($index,@{$sys->{$sysi}})) {
                push(@list, $index);
                last;
            }
        }
    }
    return \@list;
}

sub verify_all_systems {
    my $cpic=shift;
    my ($ivers,$msg,$sys,$prod,$prodname,$rel,$cv);
    return 1 unless ($cpic->{prod});
    $prod=$cpic->prod;
    $rel=$cpic->rel;
    $cpic->cli_set_title();

    for my $sys (@{$cpic->{systems}}) {
        # Note: how should this handle some systems having stop_checks set?
        next if ($sys->{stop_checks});
        $ivers=$prod->version_sys($sys);
        if (!$ivers && Cfg::opt(qw(upgrade patchupgrade))) {
            $msg=Msg::new("Cannot upgrade $prod->{abbr} to version $prod->{vers} on $sys->{sys} beacuse it is not installed on the system", 30, 1343, "$prod->{abbr}", "$prod->{vers}", "$sys->{sys}");
            $sys->push_error($msg);
        }

        $cv=EDRu::compvers($ivers,$prod->{vers},4) if ($ivers);
        if (!($sys->{errors}) && $cv && Cfg::opt(qw(upgrade patchupgrade))) {
            $prodname= $sys->{dbed_prod} || $prod->{abbr};
            $ivers = $prod->version_with_extra_mainpkgs_sys($sys);
            $msg=Msg::new("$prodname version $ivers will be upgraded to $prod->{abbr} version $prod->{vers} on $sys->{sys}", 30, 1344, "$prodname", "$ivers", "$prod->{abbr}", "$prod->{vers}", "$sys->{sys}");
            $sys->push_note($msg) unless(EDRu::inarr($prodname,@{$rel->{ru_noupgrademsg_prod}}) && (Cfg::opt('upgrade_nonkernelpkgs')|| $rel->determine_rolling_upgrade($cpic->{prod})==2));
        }
    }
    return 1;
}

sub prod_default_systemnames {
    my $cpic=shift;
    return ($cpic->{prod}) ? $cpic->prod->default_systemnames :
                             $cpic->rel->default_systemnames;
}

sub rel_platvers_sys {
    # CPIC wrapper for rel platvers check as all checks can be doced as CPIC subs
    my($cpic,$sys)=@_;
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    return '' if ($sys->{stop_checks});
    return $sys->rel->platvers_sys($sys);
}

sub prod_prerequisite_sys {
    my ($cpic,$sys) = @_;
    my (@hfpatches,$i,@ospatches,$patchobj,$rel,$msg,$vers,$ospkgs,@missing_ospkgs,@pkg_missing_oslibpkgs,$ospkg,$pkgs,@pkg_missing_oslibs,%checked_patches,$padv,@oslibs,$ospatch2,$rtns,$defvers,@missing_oslibpkgs,$oslib,@missing_oslibs,$patch,$pkgobj,$patches,$vers2,$cv,$patchi2,%checked_libs,$rtn,$pkgi,$pdfrs,@missing_ospatches,%checked_pkgs,$pkg,$ospatch,$patchi,$libs,@ospkgs,$uniq_oslibs,$prod,$minorvers);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    return 1 if ($sys->{stop_checks});
    return 1 unless (Cfg::opt(qw(install patchupgrade upgrade configure start precheck postcheck)));
    $prod=$sys->prod;
    # to have every thread right value for $prod->{pkgbucketpkgs} in upgrade
    # as in upgrade product is set during upgrade_prod_mix_check() (which take care of setting values in main thread)
    $padv=$sys->padv;
    $pdfrs=Msg::get('pdfrs');
    $minorvers=$sys->{minorvers};

    # check the existence of prod's prerequisite OS pkgs and version if any.
    @ospatches=();
    @ospkgs=();
    @oslibs=();
    for my $pkgi (@{$prod->get_pkgs}) {
        $pkg=$sys->pkg($pkgi);
        $pkg->define_ospatches($padv,$sys);
        push(@ospatches, @{$pkg->{ospatches}{all}})
            if (defined($pkg->{ospatches}{all}));
        push(@ospatches, @{$pkg->{ospatches}{$sys->patchindex()}})
            if (defined($pkg->{ospatches}{$sys->patchindex()}));
        push(@ospkgs, @{$pkg->{ospkgs}{all}})
            if (defined($pkg->{ospkgs}{all}));
        push(@ospkgs, @{$pkg->{ospkgs}{$sys->{platvers}}})
            if (defined($pkg->{ospkgs}{$sys->{platvers}}));
        push(@oslibs, keys %{$pkg->{oslibs}{all}})
            if (defined($pkg->{oslibs}{all}));
        push(@oslibs, keys %{$pkg->{oslibs}{$sys->{platvers}}})
            if (defined($pkg->{oslibs}{$sys->{platvers}}));
        if ($minorvers && defined $pkg->{oslibs}{$minorvers}) {
            push(@oslibs, keys %{$pkg->{oslibs}{$minorvers}})
        }
    }
    # Check the existence of HFs
    @hfpatches=();
    for my $pkgi (@{$prod->get_pkgs}) {
        $pkg=$sys->pkg($pkgi);
        for my $patchi (@{$pkg->{hfpatches}{$sys->{platvers}}},@{$pkg->{hfpatches}{all}}) {
            ($patchi,$vers)=split(/-/m,$patchi,2);
            $patch=$sys->patch($patchi);
            $cv=EDRu::compvers($vers,$padv->patch_version_sys($sys, $patch),4);
            push(@hfpatches,"$patchi-$vers") unless ($cv);
        }
    }
    return 1 unless (@ospatches || @ospkgs || @oslibs || @hfpatches);

    # To check OS patch dependencies.
    @missing_ospatches=();
    for my $ospatch (@ospatches) {
        next if (defined $checked_patches{$ospatch});
        $checked_patches{$ospatch}=1;

        ($patchi,$vers)=split(/-/m,$ospatch,2);
        $patchobj=Patch->new($patchi,$sys->{padv},$pkgi);
        $patchobj->{patch_vers}=$ospatch;
        push(@missing_ospatches,$ospatch) unless ($padv->ospatch_sys($sys,$patchobj));
    }

    # Code to remove lower versions of ospatches e.g remove A-A if A-B & A-A in the list (B>A)
    $i=0;
    for my $ospatch (@missing_ospatches) {
        ($patchi,$vers)=split(/-/m,$ospatch,2);
        for my $ospatch2 (@missing_ospatches) {
            next if ($ospatch eq $ospatch2);
            if ($ospatch2 =~ /$patchi/m) {
               ($patchi2,$vers2)=split(/-/m,$ospatch2,2);
               delete $missing_ospatches[$i] if (EDRu::compvers($vers2,$vers)==1);
            }
        }
        $i++;
    }

    #print the missing OS patches list
    if (@missing_ospatches) {
        $patches=join(' ',@missing_ospatches);
        if (Cfg::opt('ignorepatchreqs')) {
            $msg=Msg::new("The following required OS patches were not found on $sys->{sys}:\n\t$patches\nPackage installation may fail when required OS patches are not installed on the system", 30, 1345, "$sys->{sys}", "$patches");
            $sys->push_warning($msg);
        } else {
            $msg=Msg::new("The following required OS patches were not found on $sys->{sys}:\n\t$patches\nPackage installation may fail when required OS patches are not installed on the system. You can attempt to install $prod->{abbr} $pdfrs without the missing dependencies using the '-ignorepatchreqs' option.", 30, 1346, "$sys->{sys}", "$patches", "$prod->{abbr}", "$pdfrs");
            $sys->push_error($msg);
        }
    }

    # To check OS package dependencies.
    @missing_ospkgs=();
    for my $ospkg (@ospkgs) {
        next if (defined $checked_pkgs{$ospkg});
        $checked_pkgs{$ospkg}=1;

        ($pkgi,$defvers)=split(' ',$ospkg);
        $pkgobj=Pkg->new_pkg($pkgi, $prod->{padv});
        $vers=$pkgobj->version_sys($sys);
        if (!$vers) {
            push(@missing_ospkgs,$ospkg);
        } elsif ($defvers) {
            if (EDRu::compvers($vers,$defvers)==2) {
                $msg=Msg::new("The $pkgi version $vers is lower than $defvers on $sys->{sys}. Please upgrade the $pkgi version before you proceed.", 30, 1347, "$pkgi", "$vers", "$defvers", "$sys->{sys}", "$pkgi");
                $sys->push_warning($msg);
            }
        }
    }

    #print the missing OS pkgs list
    if (@missing_ospkgs) {
        $pkgs=join(' ',@missing_ospkgs);
        $msg=Msg::new("The following required OS $pdfrs were not found on $sys->{sys}:\n\t$pkgs", 30, 1348, "$pdfrs", "$sys->{sys}", "$pkgs");
        $sys->push_error($msg);
    }


    # To check library dependencies (mainly for Linux)
    @missing_oslibs=();
    if (@oslibs) {
        $uniq_oslibs=EDRu::arruniq(@oslibs);
        if ($padv->can('oslibraries_sys')) {
            $rtns=$padv->oslibraries_sys($sys,@{$uniq_oslibs});
            for my $i (0..$#$rtns) {
                push(@missing_oslibs,$uniq_oslibs->[$i]) unless ($rtns->[$i]);
            }
        } elsif ($padv->can('oslibrary_sys')) {
            %checked_libs=();
            for my $oslib (@{$uniq_oslibs}) {
                if (defined $checked_libs{$oslib}) {
                    $rtn=$checked_libs{$oslib};
                } else {
                    $rtn=$padv->oslibrary_sys($sys,$oslib);
                }
                $checked_libs{$oslib}=$rtn;

                push(@missing_oslibs,$oslib) unless ($rtn);
            }
        }

        # @missing_oslibs:            All missed OS libs
        # @missing_oslibpkgs:         All missed OS pkgs
        # @pkg_missing_oslibs:        Missed OS libs for one specific VRTS pkg
        # @pkg_missing_oslibpkgs:     Missed OS pkgs for one specific VRTS pkg
        if (@missing_oslibs) {
            for my $pkgi (@{$prod->get_pkgs}) {
                $pkg=$sys->pkg($pkgi);
                @oslibs=();
                push(@oslibs, keys %{$pkg->{oslibs}{all}})
                    if (defined($pkg->{oslibs}{all}));
                push(@oslibs, keys %{$pkg->{oslibs}{$sys->{platvers}}})
                    if (defined($pkg->{oslibs}{$sys->{platvers}}));
                push(@oslibs, keys %{$pkg->{oslibs}{$minorvers}})
                    if ($minorvers && defined $pkg->{oslibs}{$minorvers});
                next unless (@oslibs);

                @pkg_missing_oslibs=();
                @pkg_missing_oslibpkgs=();
                for my $oslib (@oslibs) {
                    if (EDRu::inarr($oslib,@missing_oslibs)){
                        $ospkg=$pkg->{oslibs}{all}{$oslib} || $pkg->{oslibs}{$sys->{platvers}}{$oslib} || $pkg->{oslibs}{$minorvers}{$oslib};
                        push (@pkg_missing_oslibs, $oslib);
                        if ($ospkg) {
                            push (@pkg_missing_oslibpkgs,$ospkg) unless EDRu::inarr($ospkg,@pkg_missing_oslibpkgs);
                            ($ospkg) = split(/\s+/,$ospkg);
                            push (@missing_oslibpkgs,$ospkg) unless EDRu::inarr($ospkg,@missing_oslibpkgs);
                        }
                    }
                }
                next unless (@pkg_missing_oslibs);

                $libs=join(' ', @pkg_missing_oslibs);
                $ospkgs=join(' ', @pkg_missing_oslibpkgs);
                Msg::log("The following OS libraries or $pdfrs were not found on $sys->{sys}, they are required to install $pkg->{pkg}:\n\t$libs\nIt is recommended to install the following OS packages on the system:\n\t$ospkgs");
            }

            if (@missing_oslibpkgs) {
                $ospkgs=join(' ',@missing_oslibpkgs);
                $msg=Msg::new("The following required OS $pdfrs (or higher version) were not found on $sys->{sys}:\n\t$ospkgs", 30, 1349, "$pdfrs", "$sys->{sys}", "$ospkgs");
            } else {
                $libs=join(' ',@missing_oslibs);
                $msg=Msg::new("The following required OS libraries were not found on $sys->{sys}:\n\t$libs", 30, 1350, "$sys->{sys}", "$libs");
            }
            $sys->push_error($msg);
        }
    }

    #print the installed HF patches list
    if (@hfpatches) {
        $rel=$sys->rel;
        $patches=join(' ',@hfpatches);
        $msg= Msg::new("The following Hot Fixes (HF) are found on $sys->{sys}:\n\t$patches\nThese Hot Fixes are not included in $rel->{titlevers}.Installing $prod->{abbr} $rel->{titlevers} will remove functionality provided by these hotfixes.", 30, 1351, "$sys->{sys}", "$patches", "$rel->{titlevers}", "$prod->{abbr}", "$rel->{titlevers}");
        $sys->push_warning($msg);
    }

    return '' if (@missing_ospatches || @missing_ospkgs || @missing_oslibs);
    return 1;
}

sub installed_prod_sys {
    my ($cpic,$sys) = @_;
    my ($msg,$cfg,$prodi,$ivers,$prod,$installtime,$prior_rp,$rel);
    $cfg=Obj::cfg();
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $rel=$sys->rel;
    return '' if ($sys->{stop_checks});
    $sys->pkgs_patches();
    $rel->feature_usage_sys($sys);
    $rel->check_installed_pkgs_version_sys($sys);
    for my $prodi (@{$sys->rel->{upgradeprods}}) {
        $prod=$sys->prod($prodi);
        # set $force_flag = 1 when query product version
        $ivers=$prod->version_sys($sys,1);
        $installtime = $prod->installtime_sys($sys);
        if ($ivers) {
            $sys->set_value('upgradeprod', 'push', $prodi);
            $sys->set_value('upgradeprod_abbr', 'push', $prod->{abbr});
            $sys->set_value('prodvers', 'push', $ivers);
            $sys->set_value('prodinstalltimes', 'push', $installtime);
            # replace $cpic->prod if use installsfcfs to configure SFCFSHA product
            if (!$cpic->{prod} || ($prod->configure_alternate_prod() && (Cfg::opt(qw/start stop configure precheck postcheck/))) ) {
                $cpic->set_value('prod',$prodi) unless ((Cfg::opt(qw(patchupgrade rollback))) && (EDRu::compvers($ivers,$prod->{vers},2)));
            }
        }
    }
    $cpic->set_value('prod',${$sys->{upgradeprod}}[0]) if (!$cpic->{prod} && $sys->{upgradeprod});
    if ($rel->{type} eq "RP") {
        $prior_rp = $rel->detect_prior_rp_sys($sys) if ($cpic->{prod});
        $sys->set_value('prior_rp',$prior_rp);
    }
    return 1;
}

sub sys_error {
    my ($cpic,$erronly_flag) = @_;
    for my $sys (@{$cpic->{systems}}) {
        #$erronly_flag ?
        #return 1 if ($sys->{errors}) :
        return 1 if ($sys->{errors} || $sys->{warnings});
    }
    return '';
}

sub prod_version_sys {
    my ($cpic,$sys)=@_;
    my ($prod,$prodname,$padv,$ivers,$msg,$opt,$cv,$installedprod,$rel,$noconfig_flag,$prior_rp_title);
    my ($basevers,$baseversi,@basevers_arr); 
    # noconfig_flag is related with SF or VCS partial upgrade out of installed SFHA
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $rel=$sys->rel;
    $noconfig_flag = ($sys->{noconfig_sf} || $sys->{noconfig_vcs});
    return 1 if ($sys->{stop_checks});
    return 1 if ($rel->{lp});
    if ($cpic->{prod}) {
        $prod=$sys->prod;
        $prodname=$prod->{abbr};
        $ivers=$prod->version_sys($sys);
    } else {
        $prodname = 'Any product';
    }
    if ($rel->{type} eq "RP") {        
        @basevers_arr = @{$rel->{basevers}};
        @basevers_arr = grep !/6\.0\.2/,@basevers_arr if $prodname !~ /^(VCS|SFHA)$/;
        $basevers = join ', ',@basevers_arr;
    }
    $cv=EDRu::compvers($ivers,$prod->{vers},4) if ($ivers);
    # task        check whether product installed            check whether current version
    # install     X                                          X
    # upgrade     V                                          X
    # patchupgrade     V                                     X
    # uninstall   V                                          V
    # configure   V                                          V
    # start       V                                          V
    # stop        V                                          V
    # license     V                                          V
    # precheck    X                                          X
    if (Cfg::opt(qw(uninstall rollback configure start stop startstoptest postcheck license))) {
        if (!$ivers) {
            $msg=Msg::new("$prodname $prod->{vers} does not appear to be installed on $sys->{sys}", 30, 1352, "$prodname", "$prod->{vers}", "$sys->{sys}");
            if (Cfg::opt(qw(uninstall license))) {
                $sys->push_warning($msg);
            } else {
                unless (Cfg::opt('install') && Cfg::opt('configure')) {
                    $sys->push_error($msg);
                    return '';
                }
            }
        } elsif ((${$sys->{upgradeprod}}[0] ne "") && (Cfg::opt("rollback")) && $cv) {
          # No warning message for RP as customers many times have only partial RP installed
          $msg=Msg::new("$rel->{titlevers} does not appear to be completely installed on $sys->{sys}", 30, 1353, "$rel->{titlevers}", "$sys->{sys}");
          $sys->push_warning($msg); 
        } elsif ((Cfg::opt("rollback")) && $sys->{prior_rp}) {
            $prior_rp_title = $rel->{prior_rp_title}{$sys->{prior_rp}};
            $msg=Msg::new("$sys->{sys} will be rolled back to $prior_rp_title release", 30, 1354, "$sys->{sys}", "$prior_rp_title");
            $sys->push_warning($msg);
        }
    # install/upgrade/patchupgrade/precheck below :
    } elsif (!$ivers) {
        # not installed
        if (Cfg::opt(qw(patchupgrade)) && (${$sys->{upgradeprod}}[0] eq '')) {
            return 1 if (Cfg::opt(qw(upgrade patchupgrade)) && $sys->{skip_versioncheck} && $cpic->{prod});
            $msg=Msg::new("$prodname does not appear to be installed on $sys->{sys}", 30, 1355, "$prodname", "$sys->{sys}");
            $sys->push_stop_checks_error($msg);
            return '';
        } elsif (${$sys->{upgradeprod}}[0] ne '') {
            # cross stack upgrade/install
            $installedprod=$sys->prod(${$sys->{upgradeprod}}[0]);
            $ivers=$installedprod->version_sys($sys);
            $cv=EDRu::compvers($ivers,$installedprod->{vers},4);
            if ($cv==2) {
                if (Cfg::opt('upgrade')) {
                    if (!$installedprod->upgradeable_version_sys($sys)) {
                        $msg=Msg::new("Upgrade from $installedprod->{abbr} $ivers to $prodname $prod->{vers} is not supported on $sys->{sys}", 30, 1356, "$installedprod->{abbr}", "$ivers", "$prodname", "$prod->{vers}", "$sys->{sys}");
                        $sys->push_error($msg);
                        return '';
                    }
                    if ($installedprod->zru_version_sys($sys)) {
                        $sys->set_value('zru_supported',1);
                    } else {
                        $sys->set_value('zru_supported',0);
                    }
                }
            } elsif ($cv==1) {
                # more recent version is installed
                $msg=Msg::new("A more recent version of $installedprod->{abbr}, $ivers, is already installed on $sys->{sys}", 30, 1357, "$installedprod->{abbr}", "$ivers", "$sys->{sys}");
                $sys->push_warning($msg);
            }
        } elsif (Cfg::opt(qw(install))) {
            return 1;
        }
    } elsif ($cv==0) {
        $msg=Msg::new("$prodname version $ivers is already installed on $sys->{sys}", 30, 1358, "$prodname", "$ivers", "$sys->{sys}");
        # When product already installed and user is performing RU phase 2.
        # Now RU's precheck won't call $cpic->prod_version_sys
        # And the option 'upgrade_nonkernelpkgs' will be determined after user confirmed cluster info.
        if (Cfg::opt('upgrade_nonkernelpkgs')) {
            $sys->set_value('zru_supported',1);
            return 1;
        }
        if (!$noconfig_flag) {
            if (Cfg::opt('upgrade')) {
                $sys->push_error($msg);
                return '';
            } else {
                $sys->push_warning($msg);
            }
        } else {
            if ($sys->{prod_to_upgrade} && $sys->{prod_to_upgrade}->zru_version_sys($sys)) {
                $sys->set_value('zru_supported',1);
            } else {
                $sys->set_value('zru_supported',0);
            }
        }
    } elsif (Cfg::opt("patchupgrade") && (!$rel->{pkgs})) {
        # patchonly release version check
        # block if 1. product version < target release lowest base version.
        # 2. product version > target release version.
        if (EDRu::compvers($ivers,@{$rel->{basevers}}[-1],4) == 2 ||
            EDRu::compvers($ivers,$prod->{vers},4) == 1) {
                $msg=Msg::new("$prodname $basevers does not appear to be installed on $sys->{sys}", 30, 1352, "$prodname", "$basevers", "$sys->{sys}");
                $sys->push_stop_checks_error($msg);
                return '';
        }
    } elsif ($cv==1) {
        # more recent version is installed
        $msg=Msg::new("A more recent version of $prodname, $ivers, is already installed on $sys->{sys}", 30, 1357, "$prodname", "$ivers", "$sys->{sys}");
        $sys->push_warning($msg);
    } elsif ((Cfg::opt(qw(responsefile))) &&
        (!$prod->{responsefileupgradeok})) {
        # prior version installed, can't upgrade w responsefile
        # questions being asked is almost inevitable
        # installonly may need to run start/stop/config routines to upgrade
        $opt = (Cfg::opt('responsefile')) ? '-responsefile' : '-install';
        $msg=Msg::new("$prodname version $ivers is installed on $sys->{sys}.  Upgrading $prodname is not supported using the $opt option", 30, 1359, "$prodname", "$ivers", "$sys->{sys}", "$prodname", "$opt");
        $sys->push_error($msg);
        return '';
        # Code to check if upgrade from current version is supported or not is yet to be added (UGVERS)
    } else {
        return '' if (!$sys->rel->upgradevers_sys($sys));
        if ($prod->zru_version_sys($sys)) {
            $sys->set_value('zru_supported',1);
        } else {
            $sys->set_value('zru_supported',0);
        }
    }
    return 1;
}

sub fsspace_sys {
    my ($cpic,$sys) = @_;
    my ($rootpath,$dfk,$msg,$max,@localvol,$reqspace,$plat,$n,$patch,@dfk,$free,$pdfr,@rsp,@vol,@space,$pkgi,$itv,$pkg,$patchi,@dc,$tmpdir,$prod,$tmpspace);
    my ($freeM,$maxM,$reqspaceM,$tmpspaceM);
    my $rel=Obj::rel();
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    return 1 if ($sys->{stop_checks});
    if (Cfg::opt('rootpath')) {
        $rootpath=Cfg::opt('rootpath');
        $rootpath=~s/\/+/\//mg;
        $rootpath=~s/\/$//m;
    }
    @localvol=qw(/opt /usr /var /);
    @vol=("$rootpath/opt", "$rootpath/usr", "$rootpath/var", "$rootpath/");
    $cpic->all_volumespace_sys($sys,@vol);
    $plat=EDRu::plat($sys->{padv});
    $prod=$sys->prod;

    # log free fs space
    Msg::log('Filesystem available:');
    Msg::log("System               $rootpath/opt       $rootpath/usr       $rootpath/var         $rootpath/");
    $msg=sprintf ("%-16s %10s %10s %10s %10s\n", $sys->{sys},
    $sys->{space}{"$rootpath/opt"}, $sys->{space}{"$rootpath/usr"},
    $sys->{space}{"$rootpath/var"}, $sys->{space}{"$rootpath/"});
    Msg::log($msg);

    @space=(0,0,0,0);
    $max=0;
    if ($rel->rollback_check_task()) {
        return $rel->rollback_fsspace_sys($sys);
    }
    # accumulate the required space
    Msg::log('Required filesystem space:');
    $pdfr=Msg::get('pdfr');
    $msg=sprintf('%-16s %9s %9s %9s %9s', $pdfr, $vol[0],
        $vol[1], $vol[2], $vol[3]);
    Msg::log($msg);
    # make a list of package patches, upsell pkgs are not included yet.
    for my $pkgi (@{$prod->get_pkgs}) {
        $pkg=$prod->pkg($pkgi);
        next if (Cfg::opt('upgrade_nonkernelpkgs') && (!$pkg->{unkernelpkg}));
        # mock value for pkg space
        $pkg->{space}||=[100,100,100,100];
        unless ((scalar(@{$pkg->{space}})==4) || (Cfg::opt('patchupgrade'))) {
            $msg=Msg::new("SPACE for $pkg->{pkg} is not defined on $plat", 30, 1360, "$pkg->{pkg}", "$plat");
            $sys->push_warning($msg);
            next;
        }
        unless (Cfg::opt('patchupgrade')) {
            # do not count pkg space if upsell and the pkg is already installed
            next if (($sys->pkgvers($pkg)) && (!Cfg::opt('upgrade')));
            # mock value for pkg size
            $pkg->{size}||=10;
            $max=$pkg->{size} if ($max<$pkg->{size});
            # do not count pkg space if upgrade and the pkg is already installed
            unless (($sys->pkgvers($pkg)) && (Cfg::opt('upgrade'))) {
                @space=@{$pkg->{space}};
                $msg=sprintf('%-16s %9d %9d %9d %9d', $pkg->{pkg}, $space[0],
                    $space[1], $space[2], $space[3]);
                Msg::log($msg);
                for my $n (0..3) {
                    $rsp[$n]+=$space[$n];
                }
            }
        }
        for my $patchi (@{$pkg->patches_sys($sys)}) {
            $patch=$sys->patch($patchi);
            # do not count patch space is it is already installed
            next if (($sys->linux() && $sys->{pkgvers}{"$patch->{patchname}"}) || $patch->installed_sys($sys));
            $patch->{space}||=$pkg->{space};
            unless (scalar(@{$patch->{space}})==4) {
                $msg=Msg::new("SPACE for $patch->{patchname} is not defined on $plat", 30, 1360, "$patch->{patchname}", "$plat");
                $sys->push_warning($msg);
                next;
            }
            # mock patch size
            $patch->{size}||=5;
            $max=$patch->{size} if ($max<$patch->{size});
            @space=@{$patch->{space}};
            $msg=sprintf('%-16s %9d %9d %9d %9d', $patchi, $space[0],
                $space[1], $space[2], $space[3]);
            Msg::log($msg);
            for my $n (0..3) {
                $rsp[$n]+=$space[$n];
            }
        }
    }
    $msg=sprintf('%-16s %9d %9d %9d %9d', 'KB total', $rsp[0],
       $rsp[1], $rsp[2], $rsp[3]);
    Msg::log($msg);
    # now determine file system space requirement and add
    # largest package/packages size*1.1 for copying
    $max = ($cpic->nsystems>$cpic->{threadlimit}/2) ? int($max*1.1) : int($max*3);
    $tmpdir=EDR::tmpdir();
    $dfk=$sys->cmd("_cmd_dfk $tmpdir");
    @dfk=split(/\n/,$dfk);
    $dfk=pop(@dfk);
    @dfk=split(/\s+/m,$dfk);
    # check if tmppath is set on local, mounted or rootpath file system
    $itv=EDRu::arrpos("$dfk[5]", @localvol);
    if ($itv<0) {
        $itv=EDRu::arrpos("$dfk[5]", @vol) if ($rootpath ne '');
        if ($itv<0) {
            # tmppath is set on mounted file system
            push (@vol, "$dfk[5]");
            push (@rsp, 0);
            $rsp[$#rsp]+=$max;
        } else {
            # tmppath is set on rootpath file system
            $rsp[$itv]+=$max;
        }
        Msg::log("$dfk[5] requires $max KB as the copied $pdfr and patch destination");
    } else {
        Msg::log("$localvol[$itv] requires $max KB as the copied $pdfr and patch destination");
        if (Cfg::opt('rootpath') && ($rootpath ne '')) {
            # tmppath is set on local file system and pkgs/patches are installed to rootpath
            # check local free space for gzipping/untarring
            $free=$cpic->volumespace_sys($sys, $localvol[$itv]);
            if (($free eq '') && ($itv != 3)) {
                Msg::log("$sys->{sys} does not have a unique $localvol[$itv] volume");
                $itv=3;
                $free=$cpic->volumespace_sys($sys, $localvol[$itv]);
            }
            if (($free eq '') && ($itv == 3)) {
                $msg=Msg::new("Cannot determine free space available in root file system on $sys->{sys}", 30, 1361, "$sys->{sys}");
                $sys->push_error($msg);
                return ''
            }
            if ($free<$max) {
                Msg::log("$max KB is required in the $localvol[$itv] volume and only $free KB is available on $sys->{sys}");
                $freeM = int ($free / 1024);
                $maxM = int ($max / 1024);
                $msg=Msg::new("$maxM MB is required in the $localvol[$itv] volume and only $freeM MB is available on $sys->{sys}", 30, 1362, "$maxM", "$localvol[$itv]", "$freeM", "$sys->{sys}");
                $sys->push_error($msg);
            }
        } else {
            $rsp[$itv]+=$max;
        }
    }

    # now check against free space in each volume
    for my $n (0..$#vol) {
        next if (!$rsp[$n]);
        $reqspace=$rsp[$n];
        $free=$cpic->volumespace_sys($sys, $vol[$n]);
        if ($free eq '') {
            if ($n==3) {
                if ($rootpath ne '') {
                    $msg=Msg::new("Cannot determine free space available in root ($rootpath) file system on $sys->{sys}", 30, 1363, "$rootpath", "$sys->{sys}");
                } else {
                    $msg=Msg::new("Cannot determine free space available in root file system on $sys->{sys}", 30, 1361, "$sys->{sys}");
                }
                $sys->push_error($msg);
                return '';
            }
            $rsp[3]+=$rsp[$n];
            Msg::log("$sys->{sys} does not have a unique $vol[$n] volume");
            Msg::log("$vol[$n] space of $rsp[$n] KB will be added to $vol[3]=$rsp[3]");
        } elsif ($free <$rsp[$n]) {
            $tmpspace = $rsp[$n] - $free;
            Msg::log("$reqspace KB is required in the $vol[$n] volume and only $free KB is available on $sys->{sys}. $tmpspace KB needs to be freed up in the $vol[$n] volume.");
            $reqspaceM = int ($reqspace / 1024);
            $freeM = int ($free / 1024);
            $tmpspaceM = int ($tmpspace / 1024);
            $msg=Msg::new("$reqspaceM MB is required in the $vol[$n] volume and only $freeM MB is available on $sys->{sys}. $tmpspaceM MB needs to be freed up in the $vol[$n] volume.", 30, 1364, "$reqspaceM", "$vol[$n]", "$freeM", "$sys->{sys}", "$tmpspaceM", "$vol[$n]");
            $sys->push_error($msg);
        } elsif ($free-$rsp[$n]<$cpic->{spacewarning}) {
            push(@dc,$vol[$n],$free-$rsp[$n]);
        } else {
            Msg::log("$sys->{sys} has $vol[$n]");
        }
    }
    return 1 if (Cfg::opt(qw(responsefile precheck)));
    # limited space warnings
    while (@dc) {
        $n=shift(@dc);
        $free=shift(@dc);
        $free=sprintf('%d', $free);
        $msg=Msg::new("Installing $prod->{name} will leave only approximately $free KB of free space remaining in the $n volume on $sys->{sys}", 30, 1365, "$prod->{name}", "$free", "$n", "$sys->{sys}");
        $sys->push_warning($msg);
    }
    return 1;
}

# determin the space needed for copying pkgs/patches with copy thread.
sub max_pkgssize_sys {
    my ($cpic,$sys)= @_;
    my (@pkgs,$pkg,$prod,$tmpsize,$maxsize);
    $prod=$sys->prod;
    for my $pkgi (@{$prod->get_pkgs}) {
        $pkg=$sys->pkg($pkgi);
        push (@pkgs, $pkg);
        for my $patchi (@{$pkg->patches_sys($sys)}) {
            push (@pkgs,$sys->patch($patchi));
        }
    }
    for my $i (0..($#pkgs-2)) {
        $tmpsize=int($pkgs[$i]->{size}+$pkgs[$i+1]->{size}+$pkgs[$i+2]->{size});
        $maxsize=$tmpsize if ($maxsize<$tmpsize);
    }
    return int($maxsize*1.1);
}

sub all_volumespace_sys {
    my ($cpic,$sys,@vol) = @_;
    for my $vol (@vol) {
        $sys->{space}{$vol}=$cpic->volumespace_sys($sys,$vol);
    }
    return 1;
}

sub volumespace_sys {
    my ($cpic,$sys,$vol) = @_;
    my ($dfk,$mount,@dfk);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    Msg::debug("Checking $vol volume on $sys->{sys}");
    #fix e3431209, handle encap bootdisk on HPUX on which 'bdf -l' output has multiply lines.
    for my $dfk (split(/\n/, $sys->cmd("_cmd_dfk $vol"))) {
        @dfk=split(/\s+/m, $dfk);
        $mount=$dfk[5];
        return $dfk[3] if ((defined $mount) && $vol =~ /^$mount(\/|)$/);
    }
    Msg::debug("no $vol volume on $sys->{sys}");
    return;
}

sub prod_precheck_task {
    my($cpic)= @_;
    $cpic->prod->precheck_task();
    return 1;
}

sub prod_precheck_sys {
    my($cpic,$sys)= @_;
    my($superprod,$msg);

    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    return 1 if ($sys->{stop_checks});

    # product common prerequisite check
    my $prod=$sys->prod;
    $prod->basic_precheck_sys($sys);
    $prod->rel->basic_precheck_sys($sys);

    (Cfg::opt('uninstall')) ? $prod->uninstall_precheck_sys($sys) :
        (Cfg::opt('patchupgrade')) ? $prod->patchupgrade_precheck_sys($sys) :
        (Cfg::opt('upgrade')) ? $prod->upgrade_precheck_sys($sys) :
        (Cfg::opt('rollback')) ? $prod->rollback_precheck_sys($sys) :
        (Cfg::opt('configure')) ? $prod->configure_precheck_sys($sys) :
        (Cfg::opt('start')) ? $prod->start_precheck_sys($sys) :
        (Cfg::opt('stop')) ? $prod->stop_precheck_sys($sys) :
        (Cfg::opt('postcheck')) ? $prod->postcheck_precheck_sys($sys) :
                             $prod->install_precheck_sys($sys);

    if (Cfg::opt(qw(uninstall configure stop)) && $prod->{superprods}) {
        for my $superprodi (@{$prod->{superprods}}) {
            $superprod = $prod->prod($superprodi,$sys->{padv},1);
            if ( $superprod && $superprod->version_sys($sys) ) {
                if ( Cfg::opt('uninstall')) {
                    $msg=Msg::new("$superprod->{name} is installed on $sys->{sys}. Processes or modules of $prod->{name} may fail to stop and packages of $prod->{name} may fail to uninstall", 30, 1366, "$superprod->{name}", "$sys->{sys}", "$prod->{name}", "$prod->{name}");
                } else {
                    $msg=Msg::new("$superprod->{name} is installed on $sys->{sys}. Processes or modules of $prod->{name} may fail to stop", 30, 1367, "$superprod->{name}", "$sys->{sys}", "$prod->{name}");
                }
                $sys->push_warning($msg);
                last;
            }
        }
    }

    return 1;
}

sub prod_precheck_steps {
    my($cpic)= @_;
    return $cpic->prod->precheck_steps() if ($cpic->prod->can('precheck_steps'));
    return 0;
}

sub fake_installed_packages {
    my $cpic=shift;
    my ($pkg,$pkgs,$prod,$rel);
    $prod=$cpic->prod;
    $rel=$cpic->rel;
    $pkgs=$prod->allpkgs;
    for my $pkgi (@$pkgs) {
        $pkg=$cpic->pkg($pkgi);
        for my $sys (@{$cpic->{systems}}) {
            $sys->{pkgvers}{$pkg->{pkg}}=$rel->{vers};
        }
    }
    return;
}

# the common padv can be defined per release or per-product
sub discover_common_padv {
    my $cpic=shift;
    (defined($cpic->rel->{pkgpaths})) ? $cpic->discover_rel_padv :
                                        $cpic->discover_prod_padv;
    return;
}

# for release-based settings, existence of the correct directory is enough
# this is primarily used by EEB's
sub discover_rel_padv {
    my $cpic=shift;
    my ($rel,$edr,$found,$padvisa);
    $rel=$cpic->rel;
    $edr=Obj::edr();
    for my $padvs(sort keys(%{$rel->{pkgpaths}})) {
        if (!-d "$cpic->{mediapath}/$rel->{pkgpaths}{$padvs}") {
            Msg::log("No $rel->{eebn} $padvs support: no $cpic->{mediapath}/$rel->{pkgpaths}{$padvs}");
            next;
        }
        Msg::log("$rel->{eebn} $padvs support: $cpic->{mediapath}/$rel->{pkgpaths}{$padvs} exists");
        ($padvisa,undef)=split(/\s+/, $padvs, 2);
        for my $padv(split(/\s+/, $padvs)) {
            $padv=~s/\+//;
            $padvisa=~s/\+//;
            Padv::set_padv($edr, $padv, $padvisa);
            Padv::set_padv($cpic, $padv, $padvisa);
            $cpic->{pkgpath}{$padv}=$rel->{pkgpaths}{$padvs};
            $found++;
        }
    }
    EDR::die("Did not find any padv directories") if (!$found);
    return;
}

# any to many padv for NetBackup differs between releases
# setting all potential padvs for all products and letting
# mainpkg discovery dictate exactly what to support
# should find Client for all padvs and Server/OpsCenter for
# one on release images, but this allows the potential for
# larger images with more options
sub discover_prod_padv {
    my $cpic=shift;
    my (@files,$edr,$found,$padvisa,$prod,$rel);
    $edr=Obj::edr();
    $rel=$cpic->rel;
    $prod=$cpic->prod;
    EDR::die("\%\$prod->{pkgpaths} undefined") unless (defined($prod->{pkgpaths}));
    EDR::die("\@\$prod->{mainpkgs} undefined") unless (defined($prod->{mainpkgs}));
    for my $padvs (sort keys(%{$prod->{pkgpaths}})) {
        if (!-d "$cpic->{mediapath}/$prod->{pkgpaths}{$padvs}") {
            Msg::log("No $prod->{prod} $padvs support: no $cpic->{mediapath}/$prod->{pkgpaths}{$padvs}");
            next;
        }
        for my $pkg (@{$prod->{mainpkgs}}) {
            @files = glob("$cpic->{mediapath}/$prod->{pkgpaths}{$padvs}/$pkg*");
            if (@files) {
                Msg::log("$prod->{prod} $padvs support: found $files[0]");
                ($padvisa,undef)=split(/\s+/m, $padvs, 2);
                for my $padv (split(/\s+/m, $padvs)) {
                    $padv=~s/\+//m;
                    $padvisa=~s/\+//m;
                    Padv::set_padv($edr, $padv, $padvisa);
                    Padv::set_padv($cpic, $padv, $padvisa);
                    $cpic->{pkgpath}{$padv}=$prod->{pkgpaths}{$padvs};
                    $found++;
                }
                last;
            }
        }
        Msg::log("No $prod->{prod} $padvs support: no mainpkg in $cpic->{mediapath}/$prod->{pkgpath}{$padvs}") unless (@files);
        # What about patchpath?
    }
    EDR::die("Did not find any padv directories") if (!$found);
    return;
}

sub rel_read_licenses_sys {
    my ($cpic,$sys) = @_;

    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $sys->rel->read_licenses_sys($sys);

    return 1;
}

sub handle_precheck_issues{
    my ($cpic) = @_;
    if ($cpic->{prod}) {
        $cpic->prod->handle_precheck_issues();
    }
    return 1;
}

1;
