#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package EDR;
use strict;
use Cwd qw(abs_path);

sub init_env {
    my $edr=shift;
    my ($lang,$require_file);

    # Set LANG and LC_ALL environment variables for local system.
    $lang=$ENV{LC_ALL} || $ENV{LANG} || 'C';

    # Currently, only support zh and ja on Solaris Sparc
    $lang='C' if ($lang!~/^(zh|ja)/m);
    $edr->{envlang}=$lang;

    # Set language environment to 'C'.
    $ENV{LC_ALL}='C';
    $ENV{LANG}='C';

    # handle '-require' option in the beginning.
    for my $n(0..$#ARGV) {
        next if ($ARGV[$n] ne '-require');
        $require_file=$ARGV[$n+1];
        $edr->process_require_arg($require_file);
    }

    return 1;
}

sub create_edr_objects {
    my $edr=shift;

    # create Thr object
    Thr->new() if (thread_support());

    # create Padv objects
    for my $padv (@{$edr->{padvs}}) { "Padv\::$padv"->new() if ($padv ne 'Common'); }

    return '';
}

sub set_interrupt {
    my $edr=shift;
    $SIG{INT} = $SIG{TERM} = $SIG{ABRT} = \&deal_with_sigterm;
    return;
}
 
my $terminating :shared = 0;
sub deal_with_sigterm {
    my ($edr,$msg,$web);
 
    {
        lock $terminating;
        return if ($terminating==1);
        $terminating=1;
    }
 
    if (Obj::webui()) {
        $web = Obj::web();
        $web->{interrupted}=1;
    }
 
    $edr=Obj::edr();
    $edr->{not_display_summary_file}=1;
 
    Msg::n();
    $msg=Msg::new("Interrupt Received--$edr->{script} terminated\n", 20, 1039, "$edr->{script}");
    $msg->warning;
    $msg->add_summary(1);
 
    $edr->exit_exitfile(10);
}

sub set_localsys {
    my ($edr,$hostname) = @_;
    my ($sys,$localsys);

    $localsys = $hostname || EDRu::nofqdn(`hostname`);
    $sys=$edr->{localsys}=Sys->new($localsys);
    $sys->{islocal}=1;
    $sys->{cleanup}=1;
    $sys->{plat}=$^O;
    $edr->padv_ipv_sys($sys);
    return '';
}

sub set_mediapath {
    my $edr=shift;
    my ($lcarg,$index,$mediapath,$msg);
    $index=0;
    for my $arg (@ARGV) {
        $lcarg=lc($arg);
        if ($lcarg eq '-mediapath') {
            $mediapath=$ARGV[$index+1];
            if (!defined $mediapath) {
                $msg=Msg::new("$arg requires a trailing definition argument", 20, 1040, "$arg");
                $msg->die;
            }
        }
        $index++;
    }

    $mediapath||=$edr->{mediapath};
    if ($mediapath) {
        if ($mediapath!~/^\//m) {
            $edr->{mediapath}=abs_path($edr->{scriptdir}."/$mediapath");
        } else {
            $mediapath=~s/^\/+/\//;
            $edr->{mediapath}=$mediapath;
        }
    }
    return;
}

# do whatever is necessary so that all message objects defined
# from this point on can be translated using vxgettext, if necessary
sub init_L10N {
    my $edr=shift;
    my ($padv_dir,$localsys,$localpadv,$lang);

    $lang=$edr->{envlang};

    $edr->{vx_bmcmnem}='EDR';
    $edr->{vx_bmcorigid}='9';

    if (-f "$edr->{scriptdir}/.cpi5") {
        # file marker to most effectively determine from disk installs
        $edr->{fromdisk}=1;
        $edr->{vx_bmcdomaindir}="/opt/VRTSperl/lib/site_perl/$edr->{release}";
    } else {
        $edr->{vx_bmcdomaindir}=$edr->{mediapath}.'/scripts';
    }

    $localsys=$edr->{localsys};
    $localpadv=$localsys->{padv};

    if ($localsys->linux()) {
        $padv_dir = 'linux';
        if ($localpadv=~/ppc64/m) {
            $padv_dir = 'linux_ppc64';
        }
    } elsif ($localsys->sunos()) {
        if ($localpadv=~/sparc/mi) {
            $padv_dir = 'sol_sparc';
            if (($edr->{fromdisk}) && ($lang =~ /(ja|zh)/mi)) {
                $localsys->cmd("_cmd_touch $edr->{scriptdir}/.cpi_running_l10n");
            } else {
                $localsys->cmd("_cmd_rmr $edr->{scriptdir}/.cpi_running_l10n");
            }
        } elsif ($localpadv=~/x64/m) {
            $padv_dir = 'sol_x64';
        }
    } elsif ($localsys->aix()) {
        $padv_dir = 'aix';
    }
    $edr->{vxgettext}="$edr->{vx_bmcdomaindir}/bin/$padv_dir/vxgettext" if ($padv_dir);

    EDRu::init_locale_encoding($edr);
    return '';
}

# translate some frequently used words
sub set_msgs {
    my ($edr,$msg);
    $edr=shift;

    $edr->{key}{'yes'}='y';
    $edr->{key}{'no'}='n';
    $edr->{key}{'back'}='b';
    $edr->{key}{'quit'}='q';
    $edr->{key}{'help'}='?';

    $msg=Msg::new("ERROR", 20, 1041);
    $msg->msg('error');
    $msg=Msg::new("WARNING", 20, 1042);
    $msg->msg('warning');
    $msg=Msg::new("NOTE", 20, 1043);
    $msg->msg('note');
    $msg=Msg::new("System", 20, 1044);
    $msg->msg('system');
    $edr->set_pkg_msgs();
    $edr->set_task_msgs();
    $msg=Msg::new("patch", 20, 1045);
    $msg->msg('patch');
    $msg=Msg::new("patches", 20, 1046);
    $msg->msg('patches');
    $msg=Msg::new("Back to previous menu", 20, 1047);
    $msg->msg('menuback');
    $msg=Msg::new("Press [Enter] to continue:", 20, 1048);
    $msg->msg('prtc');

    $edr->{msg}{'back'}='__back__';
    $edr->{msg}{'indent'}='    ';

    return '';
}

sub set_pkg_msgs {
    # Calling plat_sub here will default to localos and not plat()
    # Will eventually need to make this 2d for a real multi-plat train
    my ($edr,$msg,$padv);
    $edr=shift;
    $msg=Msg::new("Press [Enter] to continue:", 20, 1048);
    $msg->msg('prtc');
    $padv=${$edr->{padvs}}[0];
    if ($padv && (EDRu::plat($padv) eq 'AIX')) {
        $edr->{msg}{pdfren}='fileset';
        $msg=Msg::new("fileset", 20, 1049);
        $msg->msg('pdfr');
        $msg=Msg::new("filesets", 20, 1050);
        $msg->msg('pdfrs');
    } elsif ($padv && (EDRu::plat($padv) eq 'HPUX')) {
        $edr->{msg}{pdfren}='depot';
        $msg=Msg::new("depot", 20, 1051);
        $msg->msg('pdfr');
        $msg=Msg::new("depots", 20, 1052);
        $msg->msg('pdfrs');
    } elsif ($padv && (EDRu::plat($padv) eq 'Linux')) {
        $edr->{msg}{pdfren}='rpm';
        $msg=Msg::new("rpm", 20, 1053);
        $msg->msg('pdfr');
        $msg=Msg::new("rpms", 20, 1054);
        $msg->msg('pdfrs');
        $msg=Msg::new("Press [Enter] to continue:", 20, 1048);
        $msg->msg('prtc');
    } else {
        $edr->{msg}{pdfren}='package';
        $msg=Msg::new("package", 20, 1055);
        $msg->msg('pdfr');
        $msg=Msg::new("packages", 20, 1056);
        $msg->msg('pdfrs');
    }
    return '';
}

sub set_task_msgs {
    my ($edr,$msg);
    $edr=shift;
    $msg=Msg::new("install", 20, 1057);
    $msg->msg('task_install');
    $msg=Msg::new("uninstall", 20, 1058);
    $msg->msg('task_uninstall');
    $msg=Msg::new("configure", 20, 1059);
    $msg->msg('task_configure');
    $msg=Msg::new("upgrade", 20, 1060);
    $msg->msg('task_upgrade');
    $msg=Msg::new("stop", 20, 1061);
    $msg->msg('task_stop');
    return '';
}

sub set_tput {
    my $edr=shift;
    if (Cfg::opt('redirect')) {
        $edr->{tput}{bs}='';
        $edr->{tput}{be}='';
        $edr->{tput}{cs}='';
        $edr->{tput}{ed}='';
        $edr->{tput}{ss}='';
        $edr->{tput}{se}='';
        $edr->{tput}{us}='';
        $edr->{tput}{ue}='';
        $edr->{tput}{cl}='';
        $edr->{tput}{sc}='';
        $edr->{tput}{rc}='';
        $edr->{tput}{cuu1}='';
        $edr->{tput}{termx}=80;
        $edr->{tput}{termy} = (local_windows()) ? 0 : 24;
    } else {
        $edr->{tput}{bs}=`tput bold`;         # Bold face Start
        $edr->{tput}{be}=`tput sgr 0`;        # Bold face End
        $edr->{tput}{cs}=`tput clear`;        # Clear the Screen
        $edr->{tput}{ed}=`tput ed`;           # Clear to the display
        $edr->{tput}{ss}=`tput smso`;         # Standout face Start
        $edr->{tput}{se}=`tput rmso`;         # Standout face End
        $edr->{tput}{us}=`tput smul`;         # Underline face Start
        $edr->{tput}{ue}=`tput rmul`;         # Underline face End
        $edr->{tput}{cl}=`tput cub1`;         # cursor left
        $edr->{tput}{sc}=`tput sc`;           # save cursor pos
        $edr->{tput}{rc}=`tput rc`;           # restore cursor pos
        $edr->{tput}{cuu1}=`tput cuu 1`;      # move cursor up by 1 line
        $edr->{tput}{termx}=`tput cols`;      # number of columns
        $edr->{tput}{termy}=`tput lines`;     # number of rows
        chomp($edr->{tput}{termx});
        chomp($edr->{tput}{termy});
        $edr->{tput}{termy}||=24;
    }
    return;
}

# clears the progress status
sub set_progress_steps {
    my ($edr, $steps) = @_;

    $edr->{start} = time();
    $edr->{completed} = 0;
    $edr->{msglist} = '';
    if ($steps) {
        $edr->{steps} = $steps;
    }
    return;
}

sub init_args {
    my $edr=shift;
    my ($pdfrs,$arguments,$arg_def);

    $pdfrs=Msg::get('pdfrs');

    $edr->{arguments} = $arguments = {
        'args_def' => [ qw(mediapath require responsefile logpath tmppath uuid tunablesfile timeout) ],

        'args_opt' => [ qw(serial rsh debug nocleanup trace redirect rwe) ],

        # Those arguments have no specific entry here are considered as undocumented arguments.
        # Those arguments have specific entry here but without 'description' attribute are considered as undocumented arguments.
        # Those arguments have specific entry here and with 'undocumented' attribute are considered as undocumented arguments.

        # EDR definition args
        'mediapath' => {
            'handler' => \&EDR::process_mediapath_arg,
        },
        'responsefile' => {
            'option_description' => Msg::new("<response_file>", 20, 1062),
            'description' => Msg::new("The -responsefile option is used to perform automated installations or uninstallations using information stored in a file rather than prompting for information. <response_file> is the full path of the file that contains configuration definitions.", 20, 1063),
            'handler' => \&EDR::process_responsefile_arg,
        },
        'logpath' => {
            'option_description' => Msg::new("<log_path>", 20, 1064),
            'description' => Msg::new("The -logpath option is used to select a directory other than $edr->{logpath} as the location where $edr->{script} log files, summary file, and response file are saved", 20, 1065, "$edr->{logpath}", "$edr->{script}"),
        },
        'tmppath' => {
            'option_description' => Msg::new("<tmp_path>", 20, 1066),
            'description' => Msg::new("The -tmppath option is used to select a directory other than $edr->{tmppath} as the working directory for $edr->{script}. This destination is where initial logging is performed and where $pdfrs are copied on remote systems before installation.", 20, 1067, "$edr->{tmppath}", "$edr->{script}", "$pdfrs"),
        },
        'tunablesfile' => {
            'option_description' => Msg::new("<tunables_file>", 20, 1068),
            'description' => Msg::new("The -tunablesfile option is used to specify a tunables file including tunable parameters to be set.", 20, 1069),
            'handler' => \&EDR::process_tunablesfile_arg,
        },
        'timeout' => {
            'option_description' => Msg::new("<timeout_value>", 20, 1070),
            'description' => Msg::new("The -timeout option is used to specify the number of seconds that the script should wait for each command to complete before timing out.  Setting the -timeout option overrides the default value of 1200 seconds. Setting the -timeout option to 0 will prevent the script from timing out.  The -timeout option does not work with the -serial option", 20, 1071),
            'handler' => \&EDR::process_timeout_arg,
        },

        # EDR option args
        'require' => {
            'option_description' => Msg::new("<installer_hot_fix_file>", 20, 1072),
            'description' => Msg::new("The -require option is used to specify a installer hot fix file", 20, 1073),
        },
        'serial' => {
            'description' => Msg::new("The -serial option is used to perform install, uninstall, start, and stop operations, typically performed simultaneously on all systems, in serial fashion.", 20, 1074),
        },
        'rsh' => {
            'description' => Msg::new("The -rsh option is used to have the script use rsh and rcp for communication between the systems.  System communication using rsh and rcp is auto-detected by the script, so the -rsh option is only required when ssh and scp (default communication method) is also configured between the systems.", 20, 1075),
        },
        'redirect' => {
            'description' => Msg::new("The -redirect option is used to have the script display progress details without the use of advanced display functionality so that the output can be redirected to a file", 20, 1076),
        },
    };

    for my $arg (keys %{$arguments}) {
        $arg_def=$arguments->{$arg};
        next unless (ref($arg_def) eq 'HASH' &&
                     defined $arg_def->{handler} &&
                     !defined $arg_def->{handler_args});
        $arg_def->{handler_args} = [ $edr ];
    }
    return 1;
}

# To handle additional arguments in require file
sub init_args2 {}

sub process_args {
    my ($edr,$args_hash,$preexec)=@_;
    my ($cfg,@args,@valid_args,$args_array,$arg,$arg_def,$lcarg);
    my (@tasks,$tasks,@options,$msg,$help,$err,$handler,$handler_args);

    # initialization
    @tasks=();
    @options=();
    @valid_args=();
    for my $opt (qw(args_task args_def args_opt)) {
        $args_array=$args_hash->{$opt};
        next unless (defined $args_array);
        for my $arg (@{$args_array}) {
            push @valid_args, $arg if (!EDRu::inarr($arg,@valid_args));
        }
    }

    # analyse arguments
    $cfg=Obj::cfg();
    @args=@ARGV;
    while(@args) {
        $arg=shift(@args);
        $lcarg=lc($arg);
        if (substr($lcarg,0,1) eq '-') {
            $lcarg=substr($lcarg,1,length($lcarg));

            if ($lcarg && ($lcarg eq 'h' || $lcarg eq 'help')) {
                $help=1 if (!$preexec);
                next;
            }

            # check if valid argument
            if (!$lcarg || !EDRu::inarr($lcarg, @valid_args)) {
                next if($preexec);
                $msg=Msg::new("$arg is not a valid $edr->{script} argument", 20, 1077, "$arg", "$edr->{script}");
                $msg->warning;
                $err++;
                next;
            }

            if (EDRu::inarr($lcarg, @{$args_hash->{args_def}})) {
                if ($#args<0) {
                    $msg=Msg::new("$arg requires a trailing definition argument", 20, 1040, "$arg");
                    $msg->warning;
                    $err++;
                } else {
                    $cfg->{opt}{$lcarg}=shift(@args);
                    Msg::log("Setting cfg->{opt}{$lcarg}=$cfg->{opt}{$lcarg}");
                }
            } else {
                if (EDRu::inarr($lcarg, @{$args_hash->{args_task}})) {
                    push(@tasks, $lcarg);
                }
                $cfg->{opt}{$lcarg}=1;
                if (!EDRu::inarr($lcarg, @{$edr->{arguments}->{args_opt}})) {
                    push (@options, $lcarg);
                }
                Msg::log("Setting cfg->{opt}{$lcarg}=1");
            }
        } else {
            # other arguments are therefore system names
            push(@{$cfg->{systems}}, $arg);
            Msg::log("Adding $arg to system list");
        }
    }

    if (@tasks>1) {
        $tasks=join(' ',@tasks);
        $msg=Msg::new("The task options ($tasks) cannot be used together", 20, 1078, "$tasks");
        $msg->warning;
        $err++;
    }

    $args_hash->{tasks}=[ @tasks ] if (@tasks>1);
    $args_hash->{options}=[ @options ] if (@options>1);

    # First process those arguments with high priority, and then print help usage.
    for my $arg (@{$args_hash->{args_high_priority}}) {
        $arg_def=$args_hash->{$arg};
        next unless (defined $arg_def);
        $handler=$arg_def->{handler};
        next unless (defined $handler);
        $handler_args=$arg_def->{handler_args};
        if (defined $handler_args) {
            $err+=&$handler(@{$handler_args},$args_hash);
        } else {
            $err+=&$handler($args_hash);
        }
    }

    if ($help) {
        Msg::n() if ($err);
        $edr->usage($args_hash);
    }

    for my $arg (@valid_args) {
        # Skip those high priority args
        next if (EDRu::inarr($arg, @{$args_hash->{args_high_priority}}));

        $arg_def=$args_hash->{$arg};
        next unless (defined $arg_def);
        $handler=$arg_def->{handler};
        next unless (defined $handler);
        if (defined $cfg->{opt}{$arg}) {
            $handler_args=$arg_def->{handler_args};
            if (defined $handler_args) {
                $err+=&$handler(@{$handler_args},$args_hash);
            } else {
                $err+=&$handler($args_hash);
            }
        }
    }

    if ($err) {
        Msg::n();
        $edr->usage($args_hash);
    }

    return 1;
}

sub usage {
    my ($edr,$args_hash)=@_;
    my ($args,@tmp_args,$msg,$arg_def,$option_desc,$arg_desc,$ref_argarray);

    $msg=Msg::new("Usage: $edr->{script} [ <system1> <system2>... ]", 20, 1079, "$edr->{script}");
    $msg->print;

    exit 1 unless (defined $args_hash);

    # Show task args
    @tmp_args=();
    for my $arg (@{$args_hash->{args_task}}) {
        $arg_def=$args_hash->{$arg};
        next if (!defined $arg_def || $arg_def->{undocumented});
        push @tmp_args, $arg;
    }
    if ($#tmp_args>=0) {
        $args=join(' | -', @tmp_args);
        $msg=Msg::new("\t[ -$args ]", 20, 1080, "$args");
        $msg->print;
    }

    # Show definition args
    for my $arg (@{$args_hash->{args_def}}) {
        $arg_def=$args_hash->{$arg};
        next if (!defined $arg_def || $arg_def->{undocumented});
        $arg_desc=$arg_def->{description};
        next if (!defined $arg_desc);

        $option_desc=$arg_def->{option_description};
        if ($option_desc) {
            $option_desc=$option_desc->{msg} if (ref($option_desc) =~ m/^Msg/);
            $msg=Msg::new("\t[ -$arg $option_desc ]", 20, 1081, "$arg", "$option_desc");
        } else {
            $msg=Msg::new("\t[ -$arg ]", 20, 1080, "$arg");
        }
        $msg->print;
    }

    # Show option args
    @tmp_args=();
    for my $arg (@{$args_hash->{args_opt}}) {
        $arg_def=$args_hash->{$arg};
        next if (!defined $arg_def || $arg_def->{undocumented});
        $arg_desc=$arg_def->{description};
        next if (!defined $arg_desc);

        push @tmp_args, $arg;
    }
    if ($#tmp_args>=0) {
        $args=join(' | -', @tmp_args);
        $msg=Msg::new("\t[ -$args ]", 20, 1080, "$args");
        $msg->printn;
    }

    # Show detail help description for each argument
    $ref_argarray=EDRu::arruniq(@{$args_hash->{args_task}},
                                @{$args_hash->{args_def}},
                                @{$args_hash->{args_opt}});
    for my $arg (@{$ref_argarray}) {
        $arg_def=$args_hash->{$arg};
        next if (!defined $arg_def || $arg_def->{undocumented});
        $arg_desc=$arg_def->{description};
        next if (!defined $arg_desc);

        $arg_desc=$arg_desc->{msg} if (ref($arg_desc) =~ m/^Msg/);
        Msg::printn($arg_desc);
    }
    exit 1;
}

sub merge_args {
    my ($edr,$args_hash,$new_hash)=@_;

    for my $arg (keys %{$new_hash}) {
        if ($arg eq 'args_def') {
            push @{$args_hash->{args_def}}, @{$new_hash->{args_def}};
        } elsif ($arg eq 'args_opt') {
            push @{$args_hash->{args_opt}}, @{$new_hash->{args_opt}};
        } elsif ($arg eq 'args_task') {
            push @{$args_hash->{args_task}}, @{$new_hash->{args_task}};
        } else {
            $args_hash->{$arg}=$new_hash->{$arg};
        }
    }
    return $args_hash;
}

sub process_require_arg {
    my ($edr,$require_file)=@_;
    my ($msg,$eval_err);

    $require_file||=Cfg::opt('require');
    if ($require_file) {
        if (-f $require_file) {
            eval { require($require_file) };
            $eval_err=$@;
            if ($eval_err) {
                $eval_err =~ s/\\/\\\\/g;
                $msg=Msg::new("require file: $require_file not in correct Perl format\n\n$eval_err", 20, 1082, "$require_file", "$eval_err");
                $msg->error;
                exit 1;
            }
        } else {
            $msg=Msg::new("require file: $require_file not found", 20, 1083, "$require_file");
            $msg->error;
            exit 1;
        }
    }
    return 0;
}

sub process_mediapath_arg {
    my $edr=shift;
    my ($msg,$mediapath);

    $mediapath=Cfg::opt('mediapath');
    if (!-d $mediapath) {
        $msg=Msg::new("Specified media path $mediapath directory does not exist", 20, 1084, "$mediapath");
        $msg->die;
    }
    return 0;
}

sub process_responsefile_arg {
    my $edr=shift;
    my ($cfg,$msg,$eval_err,$nargs,$opts,$responsefile,$unsupportedargs,@unsupportedargs,$idx);

    $nargs=scalar(@ARGV);
    $cfg=Obj::cfg();
    $responsefile=$cfg->{opt}{responsefile};
    if ($responsefile) {
        if (!-f $responsefile) {
            $msg=Msg::new("responsefile file: $responsefile not found", 20, 1085, "$responsefile");
            $msg->die;
        } elsif ((($nargs>2) && !($cfg->{opt}{require}) && !($cfg->{opt}{tunablesfile})) || (($nargs>4) && (!($cfg->{opt}{require})||!($cfg->{opt}{tunablesfile}))) || ($nargs>6)) {
            # if -responsefile is used, all definitions should be in the file
            # no conflicting arguments should be allowed at the command line
            # -responsefile can be used together with -require or -tunablesfile
            @unsupportedargs = ();
            $idx = 0;
            while ($idx<$nargs) {
                if (EDRu::inarr($ARGV[$idx],qw/-require -tunablesfile -responsefile/)) {
                    $idx+=2;
                } else {
                    push @unsupportedargs, $ARGV[$idx] if (!EDRu::inarr($ARGV[$idx], @unsupportedargs));
                    $idx++;
                }
            }
            $unsupportedargs=join("\n\t",@unsupportedargs);
            if (scalar(@unsupportedargs) > 1) {
                $msg=Msg::new("-responsefile <response_file> cannot be used with the following arguments: \n\t$unsupportedargs\nThese $edr->{script} options must be defined within the response file", 20, 1086, "$unsupportedargs", "$edr->{script}");
            } else {
                $msg=Msg::new("-responsefile <response_file> cannot be used with the following argument: \n\t$unsupportedargs\nThis $edr->{script} option must be defined within the response file", 20, 1087, "$unsupportedargs", "$edr->{script}");
            }
            $msg->die;
        } else {
            our %CFG;
            our %TUN;
            eval { require($responsefile) };
            $eval_err=$@;
            if ($eval_err) {
                $eval_err =~ s/\\/\\\\/g;
                $eval_err =~ s/Compilation failed in require at.*$//;
                $msg=Msg::new("responsefile file: $responsefile not in correct Perl format\n\n$eval_err", 20, 1088, "$responsefile", "$eval_err");
                $msg->die;
            }
            $opts=$cfg->{opt};
            %$cfg = (%$cfg, %CFG);
            # merging the hashes with the line above doesn't merge $cfg{opt} correctly
            for my $opt (keys %$opts) { $cfg->{opt}{$opt}=$$opts{$opt}; }

            for my $tun (keys %TUN) {
                $cfg->{tunable}{$tun}=$TUN{$tun};
            }
        }
    }
    return 0;
}

sub process_tunablesfile_arg {
    my $edr=shift;
    my ($cfg,$msg,$eval_err,$tunablesfile);

    $cfg=Obj::cfg();
    $tunablesfile=$cfg->{opt}{tunablesfile};
    if ($tunablesfile) {
        if (!-f $tunablesfile) {
            $msg=Msg::new("tunablesfile file: $tunablesfile not found", 20, 1089, "$tunablesfile");
            $msg->die;
        } else {
            our %TUN;
            eval { require($tunablesfile) };
            $eval_err=$@;
            if ($eval_err) {
                $eval_err =~ s/\\/\\\\/g;
                $eval_err =~ s/Compilation failed in require at.*$//;
                $msg=Msg::new("tunablesfile file: $tunablesfile not in correct Perl format\n\n$eval_err", 20, 1090, "$tunablesfile", "$eval_err");
                $msg->die;
            }
            for my $tun (keys %TUN) {
                $cfg->{tunable}{$tun}=$TUN{$tun};
            }
        }
    }
    return 0;
}

sub process_timeout_arg {
    my $edr=shift;
    my ($msg,$timeout,$serial);

    $timeout=Cfg::opt('timeout');
    $serial=Cfg::opt('serial');
    if (defined $timeout) {
        if ($timeout!~/^\d+$/) {
            $msg=Msg::new("-timeout value must be an integer", 20, 1091);
            $msg->die;
        }
        if ($serial) {
            $msg=Msg::new("The -timeout option does not work with the -serial option", 20, 1092);
            $msg->die;
        }
        $edr->{timeout}=$timeout;
    }
    return 0;
}

sub create_local_tempdir {
    my $edr=shift;
    my (@lt,$cfg,$msg);
    $cfg=Obj::cfg();

    # set vars
    $edr->{tmppath}=$cfg->{opt}{tmppath} if ($cfg->{opt}{tmppath});
    $edr->{logpath}=$cfg->{opt}{logpath} if ($cfg->{opt}{logpath});
    if ($edr->{tmppath} eq $edr->{logpath}) {
       $msg=Msg::new("The -logpath and -tmppath directories must be different", 20, 1093);
       $msg->die;
    }
    @lt=localtime;
    $edr->{uuid}=$cfg->{opt}{uuid} if ($cfg->{opt}{uuid});
    $edr->{uuid}||=EDRu::basename($edr->{scriptdir}) if ($cfg->{opt}{rwe});
    $edr->{uuid}||=sprintf('%d%02d%02d%02d%02d%3s',
        $lt[5]+1900, $lt[4]+1, $lt[3], $lt[2], $lt[1], EDRu::randomstr(3));
    $edr->{scriptid}="$edr->{script}-$edr->{uuid}";
    $edr->{scriptid}=~s/\.exe-/-/;
    $edr->{tmpdir}=$edr->{localsys}->path("$edr->{tmppath}/$edr->{scriptid}");
    $edr->{logdir}=$edr->{localsys}->path("$edr->{logpath}/$edr->{scriptid}");

    # check local file system access by writing to $edr->log
    $edr->create_log_directory($edr->{logdir}, $edr->{tmpdir});
    $edr->create_log_file();
    return '';
}

sub register_save_logfiles {
    shift if (ref($_[0]) eq 'EDR');
    EDR::set_value('save_logfiles', 'push', @_);
    return;
}

sub create_and_check_directory {
    my ($edr,$dir) = @_;
    my ($m,$msg,$localsys);
    $localsys=$edr->{localsys};

    $m=$localsys->mkdir($dir);
    if (!$m || !-d $dir) {
        $msg=Msg::new("Cannot create $dir directory on $localsys->{sys}", 20, 1094, "$dir", "$localsys->{sys}");
        $msg->die();
    }
    # block non-root to access the log file
    $localsys->chmod($dir, '700');
    return;
}

sub create_log_directory {
    my ($edr,$logdir,$tmpdir) = @_;
    my ($m,$msg,$log,$localsys);
    $localsys=$edr->{localsys};

    #e3434861:check and create the path.
    $edr->create_and_check_directory($logdir);
    $edr->create_and_check_directory($tmpdir);

    # $edr->{tmpdir} is last so those values are retained
    $edr->{logfile}=$localsys->path("$tmpdir/$edr->{scriptid}.log");
    $edr->{summary}=$localsys->path("$tmpdir/$edr->{scriptid}.summary");
    $edr->{responsefile}=$localsys->path("$tmpdir/$edr->{scriptid}.response");
    $edr->{tunablesfile}=$localsys->path("$tmpdir/$edr->{scriptid}.tunables");
    $edr->{askfile}=$localsys->path("$tmpdir/$edr->{scriptid}.askfile");
    $edr->{memorydumpfile}=$localsys->path("$tmpdir/$edr->{scriptid}.json");
    $log=$edr->{logfile}.'0'; # all logs appended with thread
    $localsys->createfile($log);
    unless (-f $log) {
        $msg=Msg::new("Cannot write to $log on $localsys->{sys}", 20, 1095, "$log", "$localsys->{sys}");
        $msg->die();
    }

    register_save_logfiles("$edr->{scriptid}.log*",
                           "$edr->{scriptid}.summary",
                           "$edr->{scriptid}.response",
                           "$edr->{scriptid}.askfile",
                           "$edr->{scriptid}.json",
                          );
    return '';
}

sub create_log_file {
    no warnings 'uninitialized';
    my $edr=shift;
    Msg::log("$edr->{script} Log:");
    Msg::log("mediapath=$edr->{mediapath}");
    Msg::log("ENV{VX_BMCDOMAINDIR}=$edr->{vx_bmcdomaindir}");
    Msg::log('ARGS='.join(' ',@ARGV));
    Msg::log("localsys=$edr->{localsys}{sys}");
    Msg::log("envlang=$edr->{envlang}");
    return '';
}

sub cli_display_copyright {
    my ($edr,$year) = @_;
    my ($msg);
    Msg::title();
    $year='2014';
    $msg=Msg::new("Copyright (c) $year Symantec Corporation. All rights reserved.  Symantec, the Symantec Logo are trademarks or registered trademarks of Symantec Corporation or its affiliates in the U.S. and other countries. Other names may be trademarks of their respective owners.\n", 20, 1096, "$year");
    $msg->print;

    $msg=Msg::new("The Licensed Software and Documentation are deemed to be _Q_commercial computer software_Q_ and _Q_commercial computer software documentation_Q_ as defined in FAR Sections 12.212 and DFARS Section 227.7202.\n", 20, 1097);
    $msg->print;

    $msg=Msg::new("Logs are being written to $edr->{tmpdir} while $edr->{script} is in progress.\n", 20, 1098, "$edr->{tmpdir}", "$edr->{script}");
    $msg->print;
    return '';
}

1;
